package com.bocloud.ims.booter.controller.resource.azure.slb;

import com.bocloud.common.annotations.Log;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.resource.azure.slb.InboundNatRule;
import com.bocloud.ims.interfaces.resource.azure.slb.InboundNatRuleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

@Api(value = "入站NAT规则", tags = "入站NAT规则管理")
@RestController
@RequestMapping("/v1/nats/rules")
public class InboundNatRuleController {

	@Autowired
	private InboundNatRuleService service;

	/**
	 * 列表
	 *
	 * @param pager
	 * @return
	 */
	@ApiOperation(tags = {"CMC", "CSC"}, value = "列表入站NAT规则", httpMethod = "GET")
	@Log(module = BoCloudService.IMS, target = "入站NAT规则", operation = "列表入站NAT规则")
	@GetMapping
	public BocloudResult list(Pager pager,
							  @Value(Common.REQ_CONTEXT) RequestContext context) {
		return service.list(pager, context);
	}

	/**
	 * 详情
	 *
	 * @param id
	 * @return
	 */
	@ApiOperation(tags = {"CMC", "CSC"}, value = "详情入站NAT规则", httpMethod = "GET")
	@Log(module = BoCloudService.IMS, target = "入站NAT规则", operation = "详情入站NAT规则")
	@GetMapping("/{id}")
	public BocloudResult detail(@PathVariable(value = Common.ID) Long id,
								@Value(Common.REQ_CONTEXT) RequestContext context) {
		return service.detail(id, context);
	}


	/**
	 * 创建
	 *
	 * @param entityParams
	 * @param context
	 * @return
	 */
	@ApiOperation(tags = {"CMC", "CSC"}, value = "创建入站NAT规则", httpMethod = "POST")
	@Log(module = BoCloudService.IMS, target = "入站NAT规则", operation = "创建入站NAT规则")
	@PostMapping
	public BocloudResult create(@RequestBody InboundNatRule entityParams,
								@Value(Common.REQ_CONTEXT) RequestContext context) {
		return service.create(entityParams, context);
	}

	/**
	 * 删除
	 *
	 * @param id
	 * @param context
	 * @return
	 */
	@ApiOperation(tags = {"CMC", "CSC"}, value = "删除入站NAT规则", httpMethod = "DELETE")
	@Log(module = BoCloudService.IMS, target = "入站NAT规则", operation = "删除入站NAT规则")
	@DeleteMapping("/{id}")
	public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
								@Value(Common.REQ_CONTEXT) RequestContext context) {
		return service.remove(id, context);
	}

}
