package com.bocloud.ims.booter.controller.resource.azure.slb;

import com.bocloud.common.annotations.Log;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.resource.azure.slb.LbRule;
import com.bocloud.ims.interfaces.resource.azure.slb.LbRuleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

@Api(value = "负载均衡规则", tags = "负载均衡规则管理")
@RestController
@RequestMapping("/v1/lbs/rules")
public class LbRuleController {

    @Autowired
    private LbRuleService service;


    /**
     * 列表
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "列表负载均衡规则", httpMethod = "GET")
    @Log(module = BoCloudService.IMS, target = "负载均衡规则", operation = "列表负载均衡规则")
    @GetMapping
    public BocloudResult list(Pager pager,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        return service.list(pager, context);
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "详情负载均衡规则", httpMethod = "GET")
    @Log(module = BoCloudService.IMS, target = "负载均衡规则", operation = "详情负载均衡规则")
    @GetMapping("/{id}")
    public BocloudResult detail(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return service.detail(id, context);
    }

    /**
     * 创建
     *
     * @param entityParams
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "创建负载均衡规则", httpMethod = "POST")
    @Log(module = BoCloudService.IMS, target = "负载均衡规则", operation = "创建负载均衡规则")
    @PostMapping
    public BocloudResult create(@RequestBody LbRule entityParams,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return service.create(entityParams, context);
    }

    /**
     * 更新
     *
     * @param params
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改", httpMethod = "PUT")
    @Log(module = BoCloudService.IMS, target = "负载均衡规则", operation = "更新负载均衡规则")
    @PutMapping(value = "/{id}")
    public BocloudResult modify(@PathVariable(value = Common.ID) Long id,
                                @RequestBody LbRule params,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return service.modify(params, context);
    }

    /**
     * 删除
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除负载均衡规则", httpMethod = "DELETE")
    @Log(module = BoCloudService.IMS, target = "负载均衡规则", operation = "删除负载均衡规则")
    @DeleteMapping("/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return service.remove(id, context);
    }




}
