package com.bocloud.ims.booter.controller.resource.f5;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.ims.entity.resource.f5.BiDataGroup;
import com.bocloud.ims.interfaces.resource.f5.F5BiDataGroupService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 数据组
 *
 * @author lyy
 * @version 1.0
 * @date 2020/11/11 16:11
 */
@Api(value = "BiDataGroup", tags = "BiDataGroup")
@RestController
@RequestMapping("/v1/f5/datagroup")
public class F5BiDataGroupController {

    @Autowired
    private F5BiDataGroupService f5BiDataGroupService;

    /**
     * 列表
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看BiDataGroup列表", httpMethod = "GET")
    @GetMapping
    public BocloudResult list(Pager pager) {
        return f5BiDataGroupService.list(pager);
    }

    /**
     * 添加
     *
     * @param dataGroup
     * @param context
     * @return 添加结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加BiDataGroup", httpMethod = "POST")
    @PostMapping
    public BocloudResult create(@RequestBody BiDataGroup dataGroup,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != dataGroup) {
            return f5BiDataGroupService.create(dataGroup, context.getTarget());
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 修改
     *
     * @param dataGroup
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改BiDataGroup", httpMethod = "PUT")
    @PutMapping(value = "/{id}")
    public BocloudResult modify(@RequestBody BiDataGroup dataGroup,
                                @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != dataGroup) {
            dataGroup.setId(id);
            return f5BiDataGroupService.modify(dataGroup, context.getTarget());
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 移除
     *
     * @param id
     * @param context
     * @return 操作结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除BiDataGroup", httpMethod = "DELETE")
    @DeleteMapping(value = "/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return f5BiDataGroupService.remove(id, context.getTarget());
    }

    /**
     * 详细信息
     *
     * @param id
     * @return 操作结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询BiDataGroup详情", httpMethod = "GET")
    @GetMapping(value = "/{id}")
    public BocloudResult detail(@PathVariable(value = Common.ID) Long id) {
        return f5BiDataGroupService.detail(id);
    }
}
