package com.bocloud.ims.booter.controller.resource.f5;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.annotations.Log;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.ims.entity.resource.f5.BiIRules;
import com.bocloud.ims.interfaces.resource.f5.F5BiIRulesService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

@Api(value = "BiIRules", tags = "BiIRules")
@RestController
@RequestMapping("/v1/f5/irules")
public class F5BiIRuleController {
    @Autowired
    private F5BiIRulesService f5BiIRulesService;

    /**
     * 列表
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看BiIRules列表", httpMethod = "GET")
    @GetMapping
    public BocloudResult<GridBean<BiIRules>> list(Pager pager) {
        return f5BiIRulesService.list(pager);
    }

    /**
     * 添加
     *
     * @param biIRules
     * @param context
     * @return 添加结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加BiIRules", httpMethod = "POST")
    @PostMapping
    public BocloudResult create(@RequestBody BiIRules biIRules,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != biIRules) {
            return f5BiIRulesService.create(biIRules, context.getTarget());
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 修改
     *
     * @param biIRules
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改BiIRules", httpMethod = "PUT")
    @PutMapping(value = "/{id}")
    public BocloudResult modify(@RequestBody BiIRules biIRules,
                                @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != biIRules) {
            biIRules.setId(id);
            return f5BiIRulesService.modify(biIRules, context.getTarget());
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 移除
     *
     * @param id
     * @param context
     * @return 操作结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除BiIRules", httpMethod = "DELETE")
    @DeleteMapping(value = "/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return f5BiIRulesService.remove(id, context.getTarget());
    }

    /**
     * 详细信息
     *
     * @param id
     * @return 操作结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询BiIRules详情", httpMethod = "GET")
    @GetMapping(value = "/{id}")
    public BocloudResult<BiIRules> detail(@PathVariable(value = Common.ID) Long id) {
        return f5BiIRulesService.detail(id);
    }
}
