package com.bocloud.ims.booter.controller.resource.f5;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.annotations.Log;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.ims.entity.resource.f5.BiNode;
import com.bocloud.ims.interfaces.resource.f5.F5BiNodeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

@Api(value = "BiNode", tags = "BiNode")
@RestController
@RequestMapping("/v1/f5/nodes")
public class F5BiNodeController {
    @Autowired
    private F5BiNodeService f5BiNodeService;

    /**
     * 列表
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看BiNode列表", httpMethod = "GET")
    @GetMapping
    public BocloudResult<GridBean<BiNode>> list(Pager pager) {
        return f5BiNodeService.list(pager);
    }

    /**
     * 添加
     *
     * @param biNode
     * @param context
     * @return 添加结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加BiNode", httpMethod = "POST")
    @PostMapping
    public BocloudResult create(@RequestBody BiNode biNode,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != biNode) {
            return f5BiNodeService.create(biNode, context.getTarget());
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 修改
     *
     * @param biNode
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改BiNode", httpMethod = "PUT")
    @PutMapping(value = "/{id}")
    public BocloudResult modify(@RequestBody BiNode biNode,
                                @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != biNode) {
            biNode.setId(id);
            return f5BiNodeService.modify(biNode, context.getTarget());
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 移除
     *
     * @param id
     * @param context
     * @return 操作结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除BiNode", httpMethod = "DELETE")
    @DeleteMapping(value = "/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return f5BiNodeService.remove(id, context.getTarget());
    }

    /**
     * 详细信息
     *
     * @param id
     * @return 操作结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询BiNode详情", httpMethod = "GET")
    @GetMapping(value = "/{id}")
    public BocloudResult<BiNode> detail(@PathVariable(value = Common.ID) Long id) {
        return f5BiNodeService.detail(id);
    }

}
