package com.bocloud.ims.booter.controller.resource.f5;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.annotations.Log;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.ims.entity.resource.f5.BiPoolMember;
import com.bocloud.ims.interfaces.resource.f5.F5BiPoolMemberService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

@Api(value = "F5BiPoolMember", tags = "F5BiPoolMember")
@RestController
@RequestMapping("/v1/f5/members")
public class F5BiPoolMemberController {

    @Autowired
    private F5BiPoolMemberService biPoolMemberService;

    /**
     * 列表
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "分页查询F5BiPoolMember", httpMethod = "GET")
    @GetMapping
    public BocloudResult<GridBean<BiPoolMember>> list(Pager pager) {
        return biPoolMemberService.list(pager);
    }

    /**
     * 添加
     *
     * @param biPoolMember
     * @param context
     * @return 添加结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加PoolMember", httpMethod = "POST")
    @PostMapping
    public BocloudResult create(@RequestBody BiPoolMember biPoolMember,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != biPoolMember) {
            return biPoolMemberService.create(biPoolMember, context.getTarget());
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 修改
     *
     * @param biPoolMember
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改PoolMember", httpMethod = "PUT")
    @PutMapping(value = "/{id}")
    public BocloudResult modify(@RequestBody BiPoolMember biPoolMember,
                                @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != biPoolMember) {
            biPoolMember.setId(id);
            return biPoolMemberService.modify(biPoolMember, context.getTarget());
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 移除
     *
     * @param id
     * @param context
     * @return 操作结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "移除PoolMember", httpMethod = "DELETE")
    @DeleteMapping(value = "/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return biPoolMemberService.remove(id, context.getTarget());
    }

    /**
     * 详细信息
     *
     * @param id
     * @return 操作结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看PoolMember详情", httpMethod = "GET")
    @GetMapping(value = "/{id}")
    public BocloudResult<BiPoolMember> detail(@PathVariable(value = Common.ID) Long id) {
        return biPoolMemberService.detail(id);
    }
}
