package com.bocloud.ims.booter.controller.resource.f5;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.annotations.Log;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.ims.entity.resource.f5.BiSnatPool;
import com.bocloud.ims.interfaces.resource.f5.F5BiSnatPoolService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

@Api(value = "BiSnatPool", tags = "BiSnatPool")
@RestController
@RequestMapping("/v1/f5/snatpools")
public class F5BiSnatPoolController {
    @Autowired
    private F5BiSnatPoolService f5BiSnatPoolService;

    /**
     * 列表
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看BiSnatPool列表", httpMethod = "GET")
    @GetMapping
    public BocloudResult<GridBean<BiSnatPool>> list(Pager pager) {
        return f5BiSnatPoolService.list(pager);
    }

    /**
     * 添加
     *
     * @param biSnatPool
     * @param context
     * @return 添加结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加BiSnatPool", httpMethod = "POST")
    @PostMapping
    public BocloudResult create(@RequestBody BiSnatPool biSnatPool,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != biSnatPool) {
            return f5BiSnatPoolService.create(biSnatPool, context.getTarget());
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 修改
     *
     * @param biSnatPool
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改BiSnatPool", httpMethod = "PUT")
    @PutMapping(value = "/{id}")
    public BocloudResult modify(@RequestBody BiSnatPool biSnatPool,
                                @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != biSnatPool) {
            biSnatPool.setId(id);
            return f5BiSnatPoolService.modify(biSnatPool, context.getTarget());
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 移除
     *
     * @param id
     * @param context
     * @return 操作结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除BiSnatPool", httpMethod = "DELETE")
    @DeleteMapping(value = "/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return f5BiSnatPoolService.remove(id, context.getTarget());
    }

    /**
     * 详细信息
     *
     * @param id
     * @return 操作结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询BiSnatPool详情", httpMethod = "GET")
    @GetMapping(value = "/{id}")
    public BocloudResult<BiSnatPool> detail(@PathVariable(value = Common.ID) Long id) {
        return f5BiSnatPoolService.detail(id);
    }
}
