package com.bocloud.ims.booter.controller.resource.f5;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.annotations.Log;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.ims.entity.resource.f5.BiVirtualServer;
import com.bocloud.ims.interfaces.resource.f5.F5BiVirtualServerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

@Api(value = "BiVirtualServer", tags = "BiVirtualServer")
@RestController
@RequestMapping("/v1/f5/servers")
public class F5BiVirtualServerController {
    @Autowired
    private F5BiVirtualServerService f5BiVirtualServerService;

    /**
     * 列表
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看BiVirtualServer列表", httpMethod = "GET")
    @GetMapping
    public BocloudResult<GridBean<BiVirtualServer>> list(Pager pager) {
        return f5BiVirtualServerService.list(pager);

    }

    /**
     * 添加
     *
     * @param biVirtualServer
     * @param context
     * @return 添加结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加BiVirtualServer", httpMethod = "POST")
    @PostMapping
    public BocloudResult create(@RequestBody BiVirtualServer biVirtualServer,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != biVirtualServer) {
            return f5BiVirtualServerService.create(biVirtualServer, context.getTarget());
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 修改
     *
     * @param biVirtualServer
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改BiVirtualServer", httpMethod = "PUT")
    @PutMapping(value = "/{id}")
    public BocloudResult modify(@RequestBody BiVirtualServer biVirtualServer,
                                @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != biVirtualServer) {
            biVirtualServer.setId(id);
            return f5BiVirtualServerService.modify(biVirtualServer, context.getTarget());
        }
        return ResultTools.formatErrResult();
    }

    /**
     * 移除
     *
     * @param id
     * @param context
     * @return 操作结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除BiVirtualServer", httpMethod = "DELETE")
    @DeleteMapping(value = "/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return f5BiVirtualServerService.remove(id, context.getTarget());
    }

    /**
     * 详细信息
     *
     * @param id
     * @return 操作结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询BiVirtualServer详情", httpMethod = "GET")
    @GetMapping(value = "/{id}")
    public BocloudResult<BiVirtualServer> detail(@PathVariable(value = Common.ID) Long id) {
        return f5BiVirtualServerService.detail(id);
    }
}
