package com.bocloud.ims.booter.controller.resource.f5;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.annotations.Log;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.ims.entity.resource.f5.BiVlans;
import com.bocloud.ims.interfaces.resource.f5.F5BiVlansService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

@Api(value = "BiVlans", tags = "BiVlans")
@RestController
@RequestMapping("/v1/f5/vlans")
public class F5BiVlansController {
    @Autowired
    private F5BiVlansService f5BiVlansService;

    /**
     * 列表
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看BiVlans列表", httpMethod = "GET")
    @GetMapping
    public BocloudResult<GridBean<BiVlans>> list(Pager pager) {
        return f5BiVlansService.list(pager);
    }

    /**
     * 添加
     *
     * @param biVlans
     * @param context
     * @return 添加结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加BiVlans", httpMethod = "POST")
    @PostMapping
    public BocloudResult create(@RequestBody BiVlans biVlans,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != biVlans) {
            return f5BiVlansService.create(biVlans, context.getTarget());
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 修改
     *
     * @param biVlans
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改BiVlans", httpMethod = "PUT")
    @PutMapping(value = "/{id}")
    public BocloudResult modify(@RequestBody BiVlans biVlans,
                                @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != biVlans) {
            biVlans.setId(id);
            return f5BiVlansService.modify(biVlans, context.getTarget());
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 移除
     *
     * @param id
     * @param context
     * @return 操作结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除BiVlans", httpMethod = "DELETE")
    @DeleteMapping(value = "/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return f5BiVlansService.remove(id, context.getTarget());
    }

    /**
     * 详细信息
     *
     * @param id
     * @return 操作结果
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询BiVlans详情", httpMethod = "GET")
    @GetMapping(value = "/{id}")
    public BocloudResult<BiVlans> detail(@PathVariable(value = Common.ID) Long id) {
        return f5BiVlansService.detail(id);
    }
}
