package com.bocloud.ims.entity.resource.nsx;

import com.bocloud.common.utils.DateDeserializer;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.annotations.Column;
import com.bocloud.entity.annotations.IgnoreAll;
import com.bocloud.entity.annotations.PK;
import com.bocloud.entity.annotations.Table;
import com.bocloud.entity.bean.GenericEntity;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * nsx负载均衡器
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年3月26日
 *
 */
@NoArgsConstructor
@AllArgsConstructor
@Table("nsx_lb_service")
public class NsxLbService extends GenericEntity {

	// id
	@Setter
	@Getter
	@PK(PKStrategy.AUTO)
	private Long id;
	// 负载均衡器uuid
	@Setter
	@Getter
	@Column("lb_service_id")
	private String lbServiceId;
	// References related to this resource
	@Setter
	@Getter
	@Column("links")
	private String links;
	// Schema for this resource
	@Setter
	@Getter
	@Column("resource_schema")
	private String resourceSchema;
	// Link to this resource
	@Setter
	@Getter
	@Column("self")
	private String self;
	// Generation of this resource config
	@Setter
	@Getter
	@Column("revision")
	private Long revision;
	// 创建时间
	@Setter
	@Getter
	@JsonSerialize(using = DateSerializer.class)
	@JsonDeserialize(using = DateDeserializer.class)
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	@Column("create_time")
	private Date createTime;
	// 创建用户
	@Setter
	@Getter
	@Column("create_user")
	private String createUser;
	// 最后一次修改时间
	@Setter
	@Getter
	@JsonSerialize(using = DateSerializer.class)
	@JsonDeserialize(using = DateDeserializer.class)
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	@Column("last_modified_time")
	private Date lastModifiedTime;
	// 最后一次修改用户
	@Setter
	@Getter
	@Column("last_modified_user")
	private String lastModifiedUser;
	// Indicates protection status of this resource
	@Setter
	@Getter
	@Column("protection")
	private String protection;
	// 是否为系统资源
	@Setter
	@Getter
	@Column("system_owned")
	private Boolean systemOwned;
	// 资源类型
	@Setter
	@Getter
	@Column("resource_type")
	private String resourceType;
	// 资源标签
	@Setter
	@Getter
	@Column("tags")
	private String tags;
	// 启用日志访问
	@Setter
	@Getter
	@Column("access_log_enabled")
	private Boolean accessLogEnabled;
	// 连接
	@Setter
	@Getter
	@Column("attachment")
	private String attachment;
	// 是否启用
	@Setter
	@Getter
	@Column("enabled")
	private Boolean enabled;
	// 错误日志级别
	@Setter
	@Getter
	@Column("error_log_level")
	private String errorLogLevel;
	// 大小
	@Setter
	@Getter
	@Column("size")
	private String size;
	// 虚拟服务器id
	@Setter
	@Getter
	@Column("virtual_server_ids")
	private String virtualServerIds;
	// 云平台id
	@Setter
	@Getter
	@Column("vendor_id")
	private Long vendorId;
	// 租户id
	@Setter
	@Getter
	@Column("tenant_id")
	private Long tenantId;
	// 云平台名称
	@Setter
	@Getter
	@IgnoreAll
	private String vendorName;
	// 租户名称
	@Setter
	@Getter
	@IgnoreAll
	private String tenantName;
	// 虚拟服务器数量
	@Setter
	@Getter
	@IgnoreAll
	private Integer serverCount;
}
