package com.bocloud.ims.repository.common;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.nsx.NsxSegment;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * nsx分段
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年2月25日
 *
 */
@Repository("nsxSegmentRepository")
public class NsxSegmentRepository extends BasicGenericDao<NsxSegment, Long> {

	public NsxSegmentRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
		super(jdbcTemplate, npJdbcTemplate);
	}

	public List<NsxSegment> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = "select a.*,c.name switch_name,v.name vendor_name from nsx_segment a left join nsx_logical_switch c on c.id = a.switch_id left join cloud_vendor v on v.id = a.vendor_id where a.is_deleted = 0 ";
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		return this.list(NsxSegment.class, sql, paramMap);
	}

	public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = "select a.id,a.name from nsx_segment a where a.is_deleted = 0";
		sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		List<NsxSegment> segments = this.list(NsxSegment.class, sql, paramMap);
		List<SimpleBean> beans = new ArrayList<>();
		for (NsxSegment segment : segments) {
			beans.add(new SimpleBean(segment.getId(), segment.getName()));
		}
		return beans;
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(a.id) from nsx_segment a where a.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, params, null, "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		return this.countQuery(sql, paramMap).intValue();
	}

	public NsxSegment query(Long id) throws Exception {
		String sql = "select a.*,c.name switch_name,v.name vendor_name from nsx_segment a left join nsx_logical_switch c on c.id = a.switch_id left join cloud_vendor v on v.id = a.vendor_id where a.is_deleted = 0 and a.id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		List<NsxSegment> list = this.list(NsxSegment.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public List<NsxSegment> listByVendorId(Long vendorId) {
		String sql = "select a.*,c.name switch_name,v.name vendor_name from nsx_segment a left join nsx_logical_switch c on c.id = a.switch_id left join cloud_vendor v on v.id = a.vendor_id where a.is_deleted = 0 and a.vendor_id = :vendorId";
		Map<String, Object> paramMap = MapTools.simpleMap(Common.VENDORID, vendorId);
		List<NsxSegment> list = this.list(NsxSegment.class, sql, paramMap);
		if (list.isEmpty()) {
			return new ArrayList<>();
		}
		return list;
	}

	public boolean removeByVendorId(Long vendorId, Long menderId) {
		String sql = "update nsx_segment set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("menderId", menderId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public List<NsxSegment> listBySwitchId(Long switchId) {
		String sql = "select a.*,c.name switch_name,v.name vendor_name from nsx_segment a left join nsx_logical_switch c on c.id = a.switch_id left join cloud_vendor v on v.id = a.vendor_id where a.is_deleted = 0 and a.switch_id = :switchId";
		Map<String, Object> paramMap = MapTools.simpleMap("switchId", switchId);
		List<NsxSegment> list = this.list(NsxSegment.class, sql, paramMap);
		if (list.isEmpty()) {
			return new ArrayList<>();
		}
		return list;
	}

	public boolean removeBySwitchId(Long switchId, Long menderId) {
		String sql = "update nsx_segment set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and switch_id = :switchId";
		Map<String, Object> params = MapTools.simpleMap("switchId", switchId);
		params.put("menderId", menderId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public boolean remove(Long id, Long menderId) {
		String sql = "update nsx_segment set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		params.put("menderId", menderId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public boolean existByCondition(String column, String value, Long vendorId) throws Exception {
		String sql = "select * from nsx_segment where " + column
				+ " = :value and vendor_id = :vendorId and is_deleted = 0";
		Map<String, Object> params = MapTools.simpleMap("value", value);
		params.put(Common.VENDORID, vendorId);
		List<NsxSegment> list = this.list(NsxSegment.class, sql, params);
		if (list.isEmpty()) {
			return false;
		}
		return true;
	}

	public List<NsxSegment> listByConnectivityPath(String connectivityPath) {
		String sql = "select a.*,c.name switch_name,v.name vendor_name from nsx_segment a left join nsx_logical_switch c on c.id = a.switch_id left join cloud_vendor v on v.id = a.vendor_id where a.is_deleted = 0 and a.connectivity_path = :connectivityPath";
		Map<String, Object> paramMap = MapTools.simpleMap("connectivityPath", connectivityPath);
		List<NsxSegment> list = this.list(NsxSegment.class, sql, paramMap);
		if (list.isEmpty()) {
			return new ArrayList<>();
		}
		return list;
	}
}
