package com.bocloud.ims.repository.common;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.nsx.NsxSwitchPort;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * nsx逻辑交换机端口
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年2月25日
 *
 */
@Repository("NsxSwitchPortRepository")
public class NsxSwitchPortRepository extends BasicGenericDao<NsxSwitchPort, Long> {

	public NsxSwitchPortRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
		super(jdbcTemplate, npJdbcTemplate);
	}

	public List<NsxSwitchPort> list(int page, int rows, List<Param> params, Map<String, String> sorter)
			throws Exception {
		String sql = "select a.*,v.name vendor_name,s.name switch_name from nsx_switch_port a left join cloud_vendor v on v.id = a.vendor_id left join nsx_logical_switch s on s.id = a.switch_id where a.is_deleted = 0 ";
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		return this.list(NsxSwitchPort.class, sql, paramMap);
	}

	public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = "select a.id,a.name from nsx_switch_port a where a.is_deleted = 0";
		sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		List<NsxSwitchPort> ports = this.list(NsxSwitchPort.class, sql, paramMap);
		List<SimpleBean> beans = new ArrayList<>();
		for (NsxSwitchPort port : ports) {
			beans.add(new SimpleBean(port.getId(), port.getName()));
		}
		return beans;
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(a.id) from nsx_switch_port a where a.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, params, null, "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		return this.countQuery(sql, paramMap).intValue();
	}

	public NsxSwitchPort query(Long id) throws Exception {
		String sql = " select a.*,v.name vendor_name,s.name switch_name from nsx_switch_port a left join cloud_vendor v on v.id = a.vendor_id left join nsx_logical_switch s on s.id = a.switch_id where a.is_deleted = 0 and a.id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		List<NsxSwitchPort> list = this.list(NsxSwitchPort.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public List<NsxSwitchPort> listByVendorId(Long vendorId) {
		String sql = "select a.*,v.name vendor_name,s.name switch_name from nsx_switch_port a left join cloud_vendor v on v.id = a.vendor_id left join nsx_logical_switch s on s.id = a.switch_id where a.is_deleted = 0 and a.vendor_id = :vendorId";
		Map<String, Object> paramMap = MapTools.simpleMap(Common.VENDORID, vendorId);
		List<NsxSwitchPort> list = this.list(NsxSwitchPort.class, sql, paramMap);
		if (list.isEmpty()) {
			return new ArrayList<>();
		}
		return list;
	}

	public boolean removeByVendorId(Long vendorId, Long menderId) {
		String sql = "update nsx_switch_port set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("menderId", menderId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public List<NsxSwitchPort> listBySwitchId(Long switchId) {
		String sql = "select a.*,v.name vendor_name,s.name switch_name from nsx_switch_port a left join cloud_vendor v on v.id = a.vendor_id left join nsx_logical_switch s on s.id = a.switch_id where a.is_deleted = 0 and a.switch_id = :switchId";
		Map<String, Object> paramMap = MapTools.simpleMap("switchId", switchId);
		List<NsxSwitchPort> list = this.list(NsxSwitchPort.class, sql, paramMap);
		if (list.isEmpty()) {
			return new ArrayList<>();
		}
		return list;
	}

	public boolean removeBySwitchId(Long switchId, Long menderId) {
		String sql = "update nsx_switch_port set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and switch_id = :switchId";
		Map<String, Object> params = MapTools.simpleMap("switchId", switchId);
		params.put("menderId", menderId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public boolean remove(Long id, Long menderId) {
		String sql = "update nsx_switch_port set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		params.put("menderId", menderId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public boolean existByCondition(String column, String value, Long vendorId) throws Exception {
		String sql = "select * from nsx_switch_port where " + column
				+ " = :value and vendor_id = :vendorId and is_deleted = 0";
		Map<String, Object> params = MapTools.simpleMap("value", value);
		params.put(Common.VENDORID, vendorId);
		List<NsxSwitchPort> list = this.list(NsxSwitchPort.class, sql, params);
		if (list.isEmpty()) {
			return false;
		}
		return true;
	}

	public NsxSwitchPort getByPortId(String portId, Long vendorId) throws Exception {
		String sql = " select a.*,v.name vendor_name,s.name switch_name from nsx_switch_port a left join cloud_vendor v on v.id = a.vendor_id left join nsx_logical_switch s on s.id = a.switch_id where a.is_deleted = 0 and a.port_id = :portId and a.vendor_id = :vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("portId", portId);
		List<NsxSwitchPort> list = this.list(NsxSwitchPort.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public List<NsxSwitchPort> fwSwitchPorts(List<String> vmNames) {
		String sql = "";
		if (ListTool.isEmpty(vmNames)) {
			sql = "select * from nsx_switch_port where is_deleted = 0";
		} else {
			for (String vmName : vmNames) {
				sql += "select * from nsx_switch_port where is_deleted = 0 and name like '%" + vmName + "%' UNION ";
			}
			sql = sql.substring(0, sql.lastIndexOf("UNION"));
			sql = "select DISTINCT ct.* from (" + sql + ") ct";
		}
		return this.list(NsxSwitchPort.class, sql, new HashMap<>(8));
	}

}
