package com.bocloud.ims.repository.common;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.nsx.NsxTierOne;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * nsx tier-1路由器
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年2月25日
 *
 */
@Repository("nsxTierOneRepository")
public class NsxTierOneRepository extends BasicGenericDao<NsxTierOne, Long> {

	public NsxTierOneRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
		super(jdbcTemplate, npJdbcTemplate);
	}

	public List<NsxTierOne> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = "select a.*,t.name tenant_name,v.name vendor_name from nsx_tier_one a left join tenant t on a.tenant_id = t.id left join cloud_vendor v on v.id = a.vendor_id where a.is_deleted = 0 ";
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		return this.list(NsxTierOne.class, sql, paramMap);
	}

	public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = "select a.id,a.name from nsx_tier_one a where a.is_deleted = 0";
		sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		List<NsxTierOne> nsx_tier_ones = this.list(NsxTierOne.class, sql, paramMap);
		List<SimpleBean> beans = new ArrayList<>();
		for (NsxTierOne nsx_tier_one : nsx_tier_ones) {
			beans.add(new SimpleBean(nsx_tier_one.getId(), nsx_tier_one.getName()));
		}
		return beans;
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(a.id) from nsx_tier_one a where a.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, params, null, "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		return this.countQuery(sql, paramMap).intValue();
	}

	public NsxTierOne query(Long id) throws Exception {
		String sql = "select a.*,t.name tenant_name,v.name vendor_name from nsx_tier_one a left join tenant t on a.tenant_id = t.id left join cloud_vendor v on v.id = a.vendor_id where a.is_deleted = 0 and a.id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		List<NsxTierOne> list = this.list(NsxTierOne.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public List<NsxTierOne> listByVendorId(Long vendorId) {
		String sql = "select a.*,t.name tenant_name,v.name vendor_name from nsx_tier_one a left join tenant t on a.tenant_id = t.id left join cloud_vendor v on v.id = a.vendor_id where a.is_deleted = 0 and a.vendor_id = :vendorId";
		Map<String, Object> paramMap = MapTools.simpleMap(Common.VENDORID, vendorId);
		List<NsxTierOne> list = this.list(NsxTierOne.class, sql, paramMap);
		if (list.isEmpty()) {
			return new ArrayList<>();
		}
		return list;
	}

	public boolean removeByVendorId(Long vendorId, Long menderId) {
		String sql = "update nsx_tier_one set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("menderId", menderId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public List<NsxTierOne> listByTier0Id(Long tier0Id) {
		String sql = "select a.*,t.name tenant_name,v.name vendor_name from nsx_tier_one a left join tenant t on a.tenant_id = t.id left join cloud_vendor v on v.id = a.vendor_id where a.is_deleted = 0 and a.tier0_id = :tier0Id";
		Map<String, Object> paramMap = MapTools.simpleMap("tier0Id", tier0Id);
		List<NsxTierOne> list = this.list(NsxTierOne.class, sql, paramMap);
		if (list.isEmpty()) {
			return new ArrayList<>();
		}
		return list;
	}

	public boolean removeByTier0Id(Long tier0Id, Long menderId) {
		String sql = "update nsx_tier_one set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and tier0_id = :tier0Id";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, tier0Id);
		params.put("menderId", menderId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public boolean remove(Long id, Long menderId) {
		String sql = "update nsx_tier_one set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		params.put("menderId", menderId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public boolean existByCondition(String column, String value, Long vendorId) throws Exception {
		String sql = "select * from nsx_tier_one where " + column
				+ " = :value and vendor_id = :vendorId and is_deleted = 0";
		Map<String, Object> params = MapTools.simpleMap("value", value);
		params.put(Common.VENDORID, vendorId);
		List<NsxTierOne> list = this.list(NsxTierOne.class, sql, params);
		if (list.isEmpty()) {
			return false;
		}
		return true;
	}

	public NsxTierOne getByUuid(String uuid) throws Exception {
		String sql = "select a.*,v.name vendor_name from nsx_tier_one a left join cloud_vendor v on v.id = a.vendor_id where a.is_deleted = 0 and a.uuid = :uuid";
		Map<String, Object> params = MapTools.simpleMap("uuid", uuid);
		List<NsxTierOne> list = this.list(NsxTierOne.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

}
