package com.bocloud.ims.repository.database;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.middleware.RdsUser;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 数据库用户Repository接口实现
 *
 * @author weiwei
 */
@Repository("rdsUserRepository")
public class RdsUserRepository extends BasicGenericDao<RdsUser, Long> {

    public RdsUserRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public boolean removeByVidAndRdsId(Long vendor, Long rdsId, Long userId) throws Exception {
        String sql = "update rds_user set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where vendor_id = :vendorId and rds_id = :rdsId ";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendor);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("rdsId", rdsId);
        return this.execute(sql, params) > 0;
    }

    public List<RdsUser> listByVidAndRdsId(Long vendor, Long rdsId) throws Exception {
        String sql = "select a.* from rds_user a where a.is_deleted = 0 and a.vendor_id = :vendorId and rds_id = :rdsId ";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendor);
        params.put("rdsId", rdsId);
        return this.list(RdsUser.class, sql, params);
    }

    public List<RdsUser> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.* from rds_user a where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(RdsUser.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select b.id,b.name,b.permission from rds_user b where b.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "b");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        List<RdsUser> users = this.list(RdsUser.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<SimpleBean>();
        for (RdsUser user : users) {
            beans.add(new SimpleBean(user.getId(), user.getName()));
        }
        return beans;
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(*) from rds_user a where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public boolean remove(Long id, Long userId) {
        String sql = "update rds_user set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public RdsUser query(Long id) {
        String sql = "select a.* from rds_user a where a.is_deleted = 0 and a.id = :id";
        List<RdsUser> list = this.list(RdsUser.class, sql, MapTools.simpleMap("id", id));
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<RdsUser> list(Long rdsId) throws Exception {
        String sql = "select * from rds_user where is_deleted = 0 and rds_id = :rdsId";
        return this.list(RdsUser.class, sql, MapTools.simpleMap("rdsId", rdsId));
    }

    public boolean deleteByRds(Long rdsId) throws Exception {
        String sql = "delete from rds_user where rds_id = :rdsId";
        return this.execute(sql, MapTools.simpleMap("rdsId", rdsId)) > 0;
    }

    public RdsUser queryByUserName(String account, Long vendorId, Long rdsId) {
        String sql = "select a.* from rds_user a where a.is_deleted = 0 and a.rds_id = :rdsId and a.vendor_id = :vendorId and a.username = :account ";
        Map<String, Object> simpleMap = MapTools.simpleMap("account", account);
        simpleMap.put("vendorId", vendorId);
        simpleMap.put("rdsId", rdsId);
        List<RdsUser> list = this.list(RdsUser.class, sql, simpleMap);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

}
