package com.bocloud.ims.repository.kubernetes;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.JdbcGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.kubernetes.KubeSecret;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author weiwei
 */
@Component("kubeSecretRepository")
public class KubeSecretRepository extends JdbcGenericDao<KubeSecret, Long> {

	public KubeSecretRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
		super(jdbcTemplate, npJdbcTemplate);
	}

	public List<KubeSecret> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
		return this.list(page, rows, params, sorter, null);
	}

	public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = "select a.id,a.name from kube_secret a where a.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		List<KubeSecret> contracts = this.list(KubeSecret.class, sql, paramMap);
		List<SimpleBean> beans = new ArrayList<SimpleBean>();
		for (KubeSecret contract : contracts) {
			beans.add(new SimpleBean(contract.getId(), contract.getName()));
		}
		return beans;
	}

	public int count(List<Param> params) throws Exception {
		return this.count(params, null);
	}

	public boolean remove(Long id, Long userId) throws Exception {
		String sql = "update kube_secret set is_deleted = true , gmt_modify = :gmtModify ,mender_id = :menderId where is_deleted = 0 and id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public KubeSecret query(Long id) throws Exception {
		String sql = " select a.*, c.name vendor_name from kube_secret a left join cloud_vendor c on a.vendor_id = c.id and c.is_deleted = 0 where a.is_deleted = 0 and a.id = :id";
		Map<String, Object> param = MapTools.simpleMap(Common.ID, id);
		List<KubeSecret> list = this.list(KubeSecret.class, sql, param);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public boolean removeByVendor(Long vendorId, Long userId) throws Exception {
		String sql = "update kube_secret set is_deleted = true , gmt_modify = :gmtModify ,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public List<KubeSecret> listByVendor(Long vendorId) throws Exception {
		String sql = "select a.* from kube_secret a where a.is_deleted = 0 and a.vendor_id = :vendorId";
		Map<String, Object> param = MapTools.simpleMap(Common.VENDORID, vendorId);
		List<KubeSecret> list = this.list(KubeSecret.class, sql, param);
		return list;
	}

	public KubeSecret querByName(String serviceName, Long vendorId) throws Exception {
		String sql = "select a.* from kube_secret a where a.is_deleted = 0 and a.name = :serviceName and a.vendor_id = :vendorId  ";
		Map<String, Object> param = MapTools.simpleMap("serviceName", serviceName);
		param.put(Common.VENDORID, vendorId);
		List<KubeSecret> list = this.list(KubeSecret.class, sql, param);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public int count(List<Param> params, List<Long> projectIds) {
		String sql = "select count(1) from kube_secret a where a.is_deleted = 0";
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		if (!ListTool.isEmpty(projectIds)) {
			sql += " and a.project_id in (:projectIds) ";
			paramMap.put("projectIds", projectIds);
		}
		sql = QueryBuilder.buildRaw(sql, params, null, "a");
		return this.countQuery(sql, paramMap).intValue();
	}

	public List<KubeSecret> list(int page, int rows, List<Param> params, Map<String, String> sorter,
			List<Long> projectIds) {
		String sql = " SELECT	a.*,	c.name vendor_name,	c.type vendor_type,	t.name tenant_name FROM	kube_secret a	LEFT JOIN cloud_vendor c ON a.vendor_id = c.id	LEFT JOIN tenant t ON a.tenant_id = t.id WHERE	a.is_deleted = 0 ";
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		if (!ListTool.isEmpty(projectIds)) {
			sql += " and a.project_id in (:projectIds) ";
			paramMap.put("projectIds", projectIds);
		}
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		return this.list(KubeSecret.class, sql, paramMap);
	}

	public List<KubeSecret> listByNsId(Long nsId, Long vendorId, Long tenantId) {
		String sql = "SELECT	a.*,	c.name vendor_name,	c.type vendor_type,	t.name tenant_name FROM	kube_secret a	LEFT JOIN cloud_vendor c ON a.vendor_id = c.id	LEFT JOIN tenant t ON a.tenant_id = t.id WHERE	a.is_deleted = 0 and a.vendor_id = :vendorId ";
		Map<String, Object> param = MapTools.simpleMap(Common.VENDORID, vendorId);
		param.put(Common.VENDORID, vendorId);
		if (null != nsId && 0 != nsId) {
			sql += " and a.ns_id = :nsId  ";
			param.put("nsId", nsId);
		}
		if (null != tenantId && 0 != tenantId) {
			sql += "  and a.tenant_id = :tenantId ";
			param.put("tenantId", tenantId);
		}
		return this.list(KubeSecret.class, sql, param);
	}

}
