
package com.bocloud.ims.repository.resource;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.AutoScalingConfig;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 弹性伸缩
 * 
 * @author ymh
 * @时间 2019年11月13日
 */
@Repository("autoScalingConfigRepository")
public class AutoScalingConfigRepository extends BasicGenericDao<AutoScalingConfig, Long> {

	public AutoScalingConfigRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
		super(jdbcTemplate, npJdbcTemplate);
	}

	public List<AutoScalingConfig> list(int page, int rows, List<Param> params, Map<String, String> sorter)
			throws Exception {
		String sql = "  select a.*,i.name image_name , f.name flavor_name, k.name keypair_name from auto_scaling_config a  left join image i on a.vendor_id = i.vendor_id and a.image_uuid = i.image_uuid left join flavor f on a.vendor_id = f.vendor_id and a.flavor_uuid = f.flavor_uuid left join keypair k on a.vendor_id = k.vendor_id and a.keypair_id = k.id where a.is_deleted = 0    ";
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		return this.list(AutoScalingConfig.class, sql);
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(1) from auto_scaling_config v where v.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, params, null, "v");
		Map<String, Object> param = QueryBuilder.getParam(params);
		return this.countQuery(sql, param).intValue();
	}

	public boolean remove(Long id, Long userId) throws Exception {
		String sql = "update auto_scaling_config set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		params.put("gmtModify", new Date());
		params.put("menderId", userId);
		return this.execute(sql, params) > 0;
	}

	public AutoScalingConfig query(Long id) throws Exception {
		String sql = "  select a.*,i.name image_name , f.name flavor_name, k.name keypair_name from auto_scaling_config a  left join image i on a.vendor_id = i.vendor_id and a.image_uuid = i.image_uuid left join flavor f on a.vendor_id = f.vendor_id and a.flavor_uuid = f.flavor_uuid left join keypair k on a.vendor_id = k.vendor_id and a.keypair_id = k.id where a.is_deleted = 0  and a.id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		List<AutoScalingConfig> list = this.list(AutoScalingConfig.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public boolean removeByVid(Long vendorId, Long userId) {
		String sql = "update auto_scaling_config set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where vendor_id = :vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("gmtModify", new Date());
		params.put("menderId", userId);
		return this.execute(sql, params) > 0;
	}

}
