
package com.bocloud.ims.repository.resource;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.AutoScalingTimedTask;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 弹性伸缩
 * 
 * @author ymh
 * @时间 2019年11月13日
 */
@Repository("autoScalingTimedTaskRepository")
public class AutoScalingTimedTaskRepository extends BasicGenericDao<AutoScalingTimedTask, Long> {

	public AutoScalingTimedTaskRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
		super(jdbcTemplate, npJdbcTemplate);
	}

	public List<AutoScalingTimedTask> list(int page, int rows, List<Param> params, Map<String, String> sorter)
			throws Exception {
		String sql = " select a.* from auto_scaling_timedtask a where a.is_deleted = 0  ";
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		return this.list(AutoScalingTimedTask.class, sql);
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(1) from auto_scaling_timedtask v where v.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, params, null, "v");
		Map<String, Object> param = QueryBuilder.getParam(params);
		return this.countQuery(sql, param).intValue();
	}

	public boolean remove(Long id, Long userId) throws Exception {
		String sql = "update auto_scaling_timedtask set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		params.put("gmtModify", new Date());
		params.put("menderId", userId);
		return this.execute(sql, params) > 0;
	}

	public AutoScalingTimedTask query(Long id) throws Exception {
		String sql = "select v.* from auto_scaling_timedtask v where v.is_deleted = 0 and v.id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		List<AutoScalingTimedTask> list = this.list(AutoScalingTimedTask.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

}
