package com.bocloud.ims.repository.resource;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.CidrBlock;

import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Repository("cidrBlockRepository")
public class CidrBlockRepository extends BasicGenericDao<CidrBlock, Long> {

	public CidrBlockRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
		super(jdbcTemplate, npJdbcTemplate);
	}

	public boolean remove(Long id, Long userId) throws Exception {
		String sql = "update cidr_block set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;

	}

	public CidrBlock query(Long id) throws Exception {
		String sql = "select a.* from cidr_block a where a.is_deleted = 0 and a.id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		List<CidrBlock> list = this.list(CidrBlock.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public boolean removeByVid(Long vendorId, String regionId, Long userId, String targetId) throws Exception {
		String sql = "update cidr_block set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId and region_id = :regionId ";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("regionId", regionId);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		if(!StringUtils.isEmpty(targetId)) {
			sql += " and target_id = :targetId ";
			params.put("targetId", targetId);
		}
		return this.execute(sql, params) > 0;
	}

	public boolean removeByVid(Long vendorId, Long userId) throws Exception {
		String sql = "update cidr_block set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId ";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public List<CidrBlock> listByVid(Long vendorId, String regionId, String targetId) throws Exception {
		String sql = "select a.* from cidr_block a  where a.is_deleted = 0 and a.vendor_id = :vendorId and a.region_id = :regionId and target_id = :targetId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("targetId", targetId);
		params.put("regionId", regionId);
		return this.list(CidrBlock.class, sql, params);
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(1) from cidr_block a where a.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, params, null, "a");
		Map<String, Object> param = QueryBuilder.getParam(params);
		return this.countQuery(sql, param).intValue();
	}

	public List<CidrBlock> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = "select * from cidr_block a where a.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		return this.list(CidrBlock.class, sql, paramMap);
	}
}
