
package com.bocloud.ims.repository.resource;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.CloudStack;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * qos policy
 * 
 * @author ymh
 * @时间 2019年11月13日
 */
@Repository("cloudStackRepository")
public class CloudStackRepository extends BasicGenericDao<CloudStack, Long> {

	public CloudStackRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
		super(jdbcTemplate, npJdbcTemplate);
	}

	public List<CloudStack> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = " select a.* from cloud_stack a where a.is_deleted = 0  ";
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		return this.list(CloudStack.class, sql);
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(1) from cloud_stack v where v.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, params, null, "v");
		Map<String, Object> param = QueryBuilder.getParam(params);
		return this.countQuery(sql, param).intValue();
	}

	public boolean remove(Long id, Long userId) throws Exception {
		String sql = "update cloud_stack set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		params.put("gmtModify", new Date());
		params.put("menderId", userId);
		return this.execute(sql, params) > 0;
	}

	public List<CloudStack> listByVendorId(Long vendorId) throws Exception {
		String sql = "select a.* from cloud_stack a where a.is_deleted = 0 and a.vendor_id =:vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		return this.list(CloudStack.class, sql, params);
	}

	public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = "select a.id,a.name,a.status from cloud_stack a where a.is_deleted = 0";
		sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		List<CloudStack> volumes = this.list(CloudStack.class, sql, paramMap);
		List<SimpleBean> beans = new ArrayList<>();
		for (CloudStack qosPolicy : volumes) {
			SimpleBean simpleBean = new SimpleBean(qosPolicy.getId(), qosPolicy.getName(), qosPolicy.getStatus());
			simpleBean.setUuid(qosPolicy.getStackUuid());
			beans.add(simpleBean);
		}
		return beans;
	}

	public CloudStack getByUuid(String uuid) throws Exception {
		String sql = "select a.* from cloud_stack a where a.is_deleted = 0 and a.stack_uuid = :uuid";
		Map<String, Object> param = MapTools.simpleMap("uuid", uuid);
		List<CloudStack> list = this.list(CloudStack.class, sql, param);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public CloudStack query(Long id) throws Exception {
		String sql = "select v.* from cloud_stack v where v.is_deleted = 0 and v.id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		List<CloudStack> list = this.list(CloudStack.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public boolean removeByVid(Long vendorId, Long userId) throws Exception {
		String sql = "update cloud_stack set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where vendor_id = :vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("gmtModify", new Date());
		params.put("menderId", userId);
		return this.execute(sql, params) > 0;
	}

	public CloudStack queryByUuid(String uuid, Long vendorId) throws Exception {
		String sql = " select a.* from cloud_stack a where a.is_deleted = 0 and a.stack_uuid = :uuid and a.vendor_id = :vendorId ";
		Map<String, Object> param = MapTools.simpleMap("uuid", uuid);
		param.put(Common.VENDORID, vendorId);
		List<CloudStack> list = this.list(CloudStack.class, sql, param);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

}
