package com.bocloud.ims.repository.resource;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.HealthMonitor;
import com.google.common.collect.Lists;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 负载均衡器健康检查Repository实现s
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年9月4日
 */
@Repository("healthMonitorRepository")
public class HealthMonitorRepository extends BasicGenericDao<HealthMonitor, Long> {

	public HealthMonitorRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
		super(jdbcTemplate, npJdbcTemplate);
	}

	public List<HealthMonitor> list(int page, int rows, List<Param> params, Map<String, String> sorter)
			throws Exception {
		String sql = "select a.*,t.name tenant_name from health_monitor a left join tenant t on a.tenant_id = t.id where a.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		return this.list(HealthMonitor.class, sql, paramMap);
	}

	public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = "select a.id,a.name,a.health_uuid from health_monitor a where a.is_deleted = 0";
		sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		List<HealthMonitor> monitors = this.list(HealthMonitor.class, sql, paramMap);
		List<SimpleBean> beans = Lists.newArrayList();
		for (HealthMonitor monitor : monitors) {
			beans.add(new SimpleBean(monitor.getId(), monitor.getName(), monitor.getHealthUuid()));
		}
		return beans;
	}

	public List<HealthMonitor> listByVid(Long vendorId) throws Exception {
		String sql = "select a.*,t.name tenant_name from health_monitor a left join tenant t on a.tenant_id = t.id where a.is_deleted = 0 and a.vendor_id = :vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		return this.list(HealthMonitor.class, sql, params);
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(1) from health_monitor a where a.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, params, null, "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		return this.countQuery(sql, paramMap).intValue();
	}

	public boolean remove(Long id, Long userId) throws Exception {
		String sql = "update health_monitor set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public boolean removeByVid(Long vendorId, Long userId) throws Exception {
		String sql = "update health_monitor set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public HealthMonitor query(Long id) throws Exception {
		String sql = "select a.*,t.name tenant_name from health_monitor a left join tenant t on a.tenant_id = t.id where a.is_deleted = 0 and a.id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		List<HealthMonitor> list = this.list(HealthMonitor.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public HealthMonitor queryByUuid(String hmUuid) throws Exception {
		String sql = "select a.*,t.name tenant_name from health_monitor a left join tenant t on a.tenant_id = t.id where a.is_deleted = 0 and a.health_uuid = :hmUuid";
		Map<String, Object> params = MapTools.simpleMap("hmUuid", hmUuid);
		List<HealthMonitor> list = this.list(HealthMonitor.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public boolean removeByListenerId(Long listenerId, Long userId) throws Exception {
		String sql = "update health_monitor set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and listener_id = :listenerId";
		Map<String, Object> params = MapTools.simpleMap("listenerId", listenerId);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public HealthMonitor getByPoolId(Long poolId) throws Exception {
		String sql = "select a.*,t.name tenant_name from health_monitor a left join tenant t on a.tenant_id = t.id where a.is_deleted = 0 and a.pool_id = :poolId";
		Map<String, Object> params = MapTools.simpleMap("poolId", poolId);
		List<HealthMonitor> list = this.list(HealthMonitor.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public boolean removeByVid(Long vendorId, Long userId, String regionId) throws Exception {
		String sql = "update health_monitor set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0  and region_id = :regionId and vendor_id = :vendorId";
		Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		params.put("regionId", regionId);
		return this.execute(sql, params) > 0;
	}

	public List<HealthMonitor> listByVid(Long vendorId, String regionId) throws Exception {
		String sql = "select a.* from health_monitor a where a.is_deleted = 0 and a.vendor_id = :vendorId and a.region_id = :regionId ";
		Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
		params.put("regionId", regionId);
		return this.list(HealthMonitor.class, sql, params);
	}

	public boolean removeByPoolId(Long poolId, Long userId) throws Exception {
		String sql = "update health_monitor set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and pool_id = :poolId";
		Map<String, Object> params = MapTools.simpleMap("poolId", poolId);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

}
