package com.bocloud.ims.repository.resource;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.InternetGateway;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Repository("internetGatewayRepository")
public class InternetGatewayRepository extends BasicGenericDao<InternetGateway, Long> {

    public InternetGatewayRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from internet_gateway a where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public List<InternetGateway> list(int page, int rows, List<Param> params, Map<String, String> sorter)
            throws Exception {
        String sql = "select a.*,r.name region_name from internet_gateway a"
                + " left join region r on r.vendor_id = a.vendor_id and r.region_id = a.region_id "
                + " where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(InternetGateway.class, sql, paramMap);
    }

    public boolean remove(Long id, Long userId) throws Exception {
        String sql = "update internet_gateway set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public InternetGateway query(Long id) throws Exception {
        String sql = "select a.* from internet_gateway a where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<InternetGateway> list = this.list(InternetGateway.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<InternetGateway> listByVid(Long vendorId) throws Exception {
        String sql = "select a.*,v.name vendor_name from internet_gateway a left join cloud_vendor v on v.id = a.vendor_id where a.is_deleted = 0 and a.vendor_id = :vendorId";
        Map<String, Object> paramMap = MapTools.simpleMap(Common.VENDORID, vendorId);
        return this.list(InternetGateway.class, sql, paramMap);
    }

    public List<InternetGateway> listByVid(Long vendorId, String regionId) throws Exception {
        String sql = "select a.*,v.name vendor_name from internet_gateway a left join cloud_vendor v on v.id = a.vendor_id where a.is_deleted = 0 and a.region_id = :regionId and a.vendor_id = :vendorId";
        Map<String, Object> paramMap = MapTools.simpleMap(Common.VENDORID, vendorId);
        paramMap.put("regionId", regionId);
        return this.list(InternetGateway.class, sql, paramMap);
    }

    public Boolean removeByVid(Long vendorId, Long userId) throws Exception {
        String sql = "update internet_gateway set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId";
        Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public Boolean removeByVid(Long vendorId, Long userId, String regionId) throws Exception {
        String sql = "update internet_gateway set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and region_id = :regionId and vendor_id = :vendorId";
        Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
        params.put("regionId", regionId);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public InternetGateway getByVpcId(String vpcId, Long vendorId, String regionId) throws Exception {
        String sql = "select a.* from internet_gateway a where a.is_deleted = 0 and a.vpc_id = :vpcId and a.vendor_id = :vendorId and a.region_id = :regionId";
        Map<String, Object> params = MapTools.simpleMap("vpcId", vpcId);
        params.put(Common.VENDORID, vendorId);
        params.put(Common.REGIONID, regionId);
        List<InternetGateway> list = this.list(InternetGateway.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<InternetGateway> getByVpc(String vpcId, Long vendorId) throws Exception {
        String sql = "select a.* from internet_gateway a where a.is_deleted = 0 and a.vpc_id = :vpcId and a.vendor_id = :vendorId";
        Map<String, Object> paramMap = MapTools.simpleMap("vpcId", vpcId);
        paramMap.put(Common.VENDORID, vendorId);
        return this.list(InternetGateway.class, sql, paramMap);
    }


}
