package com.bocloud.ims.repository.resource;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.LbPool;
import com.google.common.collect.Lists;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 负载均衡器成员池Repository接口实现
 *
 * @author lyy
 * @Version 1.0
 * @since 2017年9月4日
 */
@Repository("lbPoolRepository")
public class LbPoolRepository extends BasicGenericDao<LbPool, Long> {

    public LbPoolRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<LbPool> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.*,t.name tenant_name from lb_pool a left join tenant t on a.tenant_id = t.id where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(LbPool.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.id,a.name,a.pool_uuid from lb_pool a where a.is_deleted = 0";
        sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        List<LbPool> pools = this.list(LbPool.class, sql, paramMap);
        List<SimpleBean> beans = Lists.newArrayList();
        for (LbPool pool : pools) {
            beans.add(new SimpleBean(pool.getId(), pool.getName(), pool.getPoolUuid()));
        }
        return beans;
    }

    public List<LbPool> listByVid(Long vendorId) throws Exception {
        String sql = "select a.*,t.name tenant_name from lb_pool a left join tenant t on a.tenant_id = t.id where a.is_deleted = 0 and a.vendor_id = :vendorId";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        return this.list(LbPool.class, sql, params);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from lb_pool a where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public boolean remove(Long id, Long userId) throws Exception {
        String sql = "update lb_pool set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public boolean removeByVid(Long vendorId, Long userId) throws Exception {
        String sql = "update lb_pool set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public LbPool query(Long id) throws Exception {
        String sql = "select a.*,t.name tenant_name from lb_pool a left join tenant t on a.tenant_id = t.id where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<LbPool> list = this.list(LbPool.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public boolean removeByListenerId(Long listenerId, Long userId) throws Exception {
        String sql = "update lb_pool set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and listener_id = :listenerId";
        Map<String, Object> params = MapTools.simpleMap("listenerId", listenerId);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public List<LbPool> listByListenerId(Long listenerId) throws Exception {
        String sql = "select a.*,t.name tenant_name from lb_pool a left join tenant t on a.tenant_id = t.id where a.is_deleted = 0 and a.listener_id = :listenerId";
        Map<String, Object> params = MapTools.simpleMap("listenerId", listenerId);
        return this.list(LbPool.class, sql, params);
    }

    public boolean resetHealthMonitor(Long vendorId) throws Exception {
        String sql = "update lb_pool set hm_id = null,hm_uuid = null where vendor_id = :vendorId and is_deleted = 0";
        Map<String, Object> paramMap = MapTools.simpleMap("vendorId", vendorId);
        return this.execute(sql, paramMap) > 0;
    }

    public boolean resetHmIdAndUuid(Long hMonitorId) throws Exception {
        String sql = "update lb_pool set hm_id = null,hm_uuid = null where hm_id = :hMonitorId and is_deleted = 0";
        Map<String, Object> paramMap = MapTools.simpleMap("hMonitorId", hMonitorId);
        return this.execute(sql, paramMap) > 0;
    }

    public boolean updateHmByPoolId(Long poolId, Long hMonitorId, String hMonitorUuid) throws Exception {
        String sql = "update lb_pool set hm_id = :hMonitorId,hm_uuid = :hMonitorUuid where id = :poolId and is_deleted = 0";
        Map<String, Object> paramMap = MapTools.simpleMap("poolId", poolId);
        paramMap.put("hMonitorId", hMonitorId);
        paramMap.put("hMonitorUuid", hMonitorUuid);
        return this.execute(sql, paramMap) > 0;
    }

    public LbPool queryByUuid(String poolUuid, Long vendorId) throws Exception {
        String sql = "select a.*,t.name tenant_name from lb_pool a left join tenant t on a.tenant_id = t.id where a.is_deleted = 0 and a.pool_uuid = :poolUuid " +
                " and a.vendor_id = :vendorId";
        Map<String, Object> params = MapTools.simpleMap("poolUuid", poolUuid);
        params.put("vendorId", vendorId);
        List<LbPool> list = this.list(LbPool.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public LbPool getByListenerId(Long listenerId) throws Exception {
        String sql = "select a.*,t.name tenant_name from lb_pool a left join tenant t on a.tenant_id = t.id where a.is_deleted = 0 and a.listener_id = :listenerId";
        Map<String, Object> params = MapTools.simpleMap("listenerId", listenerId);
        List<LbPool> list = this.list(LbPool.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public boolean removeByVid(Long vendorId, Long userId, String regionId) throws Exception {
        String sql = "update lb_pool set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and region_id = :regionId and vendor_id = :vendorId";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        params.put("regionId", regionId);
        return this.execute(sql, params) > 0;
    }

    public List<LbPool> listByVid(Long vendorId, String regionId) throws Exception {
        String sql = "select a.* from lb_pool a  where a.is_deleted = 0 and a.vendor_id = :vendorId and a.region_id = :regionId ";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        params.put("regionId", regionId);
        return this.list(LbPool.class, sql, params);
    }

    public List<LbPool> listByLbId(Long lbId) throws Exception {
        String sql = "select a.* from lb_pool a  where a.is_deleted = 0 and a.lb_id = :lbId";
        Map<String, Object> params = MapTools.simpleMap("lbId", lbId);
        return this.list(LbPool.class, sql, params);
    }

    public boolean removeByLbId(Long lbId, Long userId) throws Exception {
        String sql = "update lb_pool set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and lb_id = :lbId ";
        Map<String, Object> params = MapTools.simpleMap("lbId", lbId);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }
}
