package com.bocloud.ims.repository.resource;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.LbTarget;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Repository("lbTargetRepository")
public class LbTargetRepository extends BasicGenericDao<LbTarget, Long> {

	public LbTargetRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
		super(jdbcTemplate, npJdbcTemplate);
	}

	public List<LbTarget> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = "select a.*,r.name region_name from lb_target a left join region r on r.vendor_id = a.vendor_id and r.region_id = a.region_id where a.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		return this.list(LbTarget.class, sql, paramMap);
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(1) from lb_target a where a.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, params, null, "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		return this.countQuery(sql, paramMap).intValue();
	}

	public boolean remove(Long id, Long userId) throws Exception {
		String sql = "update lb_target set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;

	}

	public LbTarget query(Long id) throws Exception {
		String sql = "select a.* from lb_target a where a.is_deleted = 0 and a.id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		List<LbTarget> list = this.list(LbTarget.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public LbTarget queryRegistered(Long tgId, String uuid, Integer port) throws Exception {
		String sql = "select a.* from lb_target a where a.is_deleted = 0 and a.tg_id = :tgId and a.uuid = :uuid and a.port = :port";
		Map<String, Object> params = MapTools.simpleMap("tgId", tgId);
		params.put("uuid", uuid);
		params.put("port", port);
		List<LbTarget> list = this.list(LbTarget.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public boolean removeByVid(Long vendorId, String regionId, Long userId) throws Exception {
		String sql = "update lb_target set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId and region_id = :regionId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("regionId", regionId);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public boolean removeByTgId(Long tgId, Long userId) throws Exception {
		String sql = "update lb_target set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and tg_id = :tgId";
		Map<String, Object> params = MapTools.simpleMap("tgId", tgId);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public List<LbTarget> listByVid(Long vendorId, String regionId) throws Exception {
		String sql = "select a.* from lb_target a  where a.is_deleted = 0 and a.vendor_id = :vendorId and a.region_id = :regionId ";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("regionId", regionId);
		return this.list(LbTarget.class, sql, params);
	}

	public List<LbTarget> list(Long tgId) throws Exception {
		String sql = "select a.* from lb_target a where a.is_deleted = 0 and a.tg_id = :tgId";
		Map<String, Object> params = MapTools.simpleMap("tgId", tgId);
		return this.list(LbTarget.class, sql, params);
	}
}
