
package com.bocloud.ims.repository.resource;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.QosPolicyRules;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * qos policy rules
 * 
 * @author ymh
 * @时间 2019年11月13日
 */
@Repository("qosPolicyRulesRepository")
public class QosPolicyRulesRepository extends BasicGenericDao<QosPolicyRules, Long> {

	public QosPolicyRulesRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
		super(jdbcTemplate, npJdbcTemplate);
	}

	public List<QosPolicyRules> list(int page, int rows, List<Param> params, Map<String, String> sorter)
			throws Exception {
		String sql = " select a.* from qos_policy_rules a where a.is_deleted = 0 ";
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		return this.list(QosPolicyRules.class, sql);
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(1) from qos_policy_rules v where v.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, params, null, "v");
		Map<String, Object> param = QueryBuilder.getParam(params);
		return this.countQuery(sql, param).intValue();
	}

	public boolean remove(Long id, Long userId) throws Exception {
		String sql = "update qos_policy_rules set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		params.put("gmtModify", new Date());
		params.put("menderId", userId);
		return this.execute(sql, params) > 0;
	}

	public boolean remove(String policyUuid, Long vendorId, Long userId) throws Exception {
		String sql = "update qos_policy_rules set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where policy_uuid = :policyUuid and vendor_id = :vendorId ";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("gmtModify", new Date());
		params.put("policyUuid", policyUuid);
		params.put("menderId", userId);
		return this.execute(sql, params) > 0;
	}

	public List<QosPolicyRules> listByVendorId(Long vendorId) throws Exception {
		String sql = "select a.* from qos_policy_rules a where a.is_deleted = 0 and a.vendor_id =:vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		return this.list(QosPolicyRules.class, sql, params);
	}

	public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = "select a.id,a.name,a.status,a.rule_uuid from qos_policy_rules a where a.is_deleted = 0";
		sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		List<QosPolicyRules> qosPolicyRules = this.list(QosPolicyRules.class, sql, paramMap);
		List<SimpleBean> beans = new ArrayList<>();
		for (QosPolicyRules rule : qosPolicyRules) {
			SimpleBean simpleBean = new SimpleBean(rule.getId(), rule.getName(), rule.getStatus());
			simpleBean.setUuid(rule.getRuleUuid());
			beans.add(simpleBean);
		}
		return beans;
	}

	public QosPolicyRules getByRuleUuid(String ruleUuid) throws Exception {
		String sql = "select a.* from qos_policy_rules a where a.is_deleted = 0 and a.rule_uuid = :ruleUuid";
		Map<String, Object> param = MapTools.simpleMap("ruleUuid", ruleUuid);
		List<QosPolicyRules> list = this.list(QosPolicyRules.class, sql, param);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public QosPolicyRules query(Long id) throws Exception {
		String sql = "select v.* from qos_policy_rules v where v.is_deleted = 0 and v.id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		List<QosPolicyRules> list = this.list(QosPolicyRules.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public boolean removeByVid(Long vendorId, Long userId) throws Exception {
		String sql = "update qos_policy_rules set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where vendor_id = :vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("gmtModify", new Date());
		params.put("menderId", userId);
		return this.execute(sql, params) > 0;
	}

}
