package com.bocloud.ims.repository.resource;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.RouteTableAssociation;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Repository("routeTableAssociationRepository")
public class RouteTableAssociationRepository extends BasicGenericDao<RouteTableAssociation, Long> {

    public RouteTableAssociationRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<RouteTableAssociation> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.*,r.name region_name from route_table_association a left join region r on r.vendor_id = a.vendor_id and r.region_id = a.region_id where a.is_deleted = 0";
        Map<String, Object> dataMap = new HashMap<>();
        for (Param param : params) {
            dataMap = param.getParam();
            if (dataMap.containsKey("subnetUuid")) {
                if ("IS NOT NULL".equals(dataMap.get("subnetUuid"))) {
                    dataMap.remove("subnetUuid");
                    sql += " and a.subnet_uuid IS NOT NULL";
                }
                break;
            }
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (sql.contains(" and a.subnet_uuid IS NOT NULL")) {
            dataMap.put("subnetUuid", "IS NOT NULL");
        }
        return this.list(RouteTableAssociation.class, sql, paramMap);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from route_table_association a where a.is_deleted = 0";
        Map<String, Object> dataMap = new HashMap<>();
        for (Param param : params) {
            dataMap = param.getParam();
            if (dataMap.containsKey("subnetUuid")) {
                if ("IS NOT NULL".equals(dataMap.get("subnetUuid"))) {
                    dataMap.remove("subnetUuid");
                    sql += " and a.subnet_uuid IS NOT NULL";
                }
                break;
            }
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (sql.contains(" and a.subnet_uuid IS NOT NULL")) {
            dataMap.put("subnetUuid", "IS NOT NULL");
        }
        return this.countQuery(sql, paramMap).intValue();
    }

    public boolean remove(Long id, Long userId) throws Exception {
        String sql = "update route_table_association set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;

    }

    public RouteTableAssociation query(Long id) throws Exception {
        String sql = "select a.* from route_table_association a where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<RouteTableAssociation> list = this.list(RouteTableAssociation.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public RouteTableAssociation queryByMain(Long vpcId, Long vendorId, String regionId) throws Exception {
        String sql = "select a.* from route_table_association a where a.is_deleted = 0 and a.vendor_id = :vendorId and a.region_id = :regionId and a.main = 1 and a.route_table_uuid in (select uuid from route_table where vpc_id = :vpcId)";
        Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
        params.put("regionId", regionId);
        params.put("vpcId", vpcId);
        List<RouteTableAssociation> list = this.list(RouteTableAssociation.class, sql, params);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }


    public boolean removeByVid(Long vendorId, String regionId, Long userId) throws Exception {
        String sql = "update route_table_association set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId and region_id = :regionId";
        Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
        params.put("regionId", regionId);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public List<RouteTableAssociation> listByVid(Long vendorId, String regionId) throws Exception {
        String sql = "select a.* from route_table_association a  where a.is_deleted = 0 and a.vendor_id = :vendorId and a.region_id = :regionId ";
        Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
        params.put("regionId", regionId);
        return this.list(RouteTableAssociation.class, sql, params);
    }

    public boolean removeByVid(Long vendorId, String regionId, Long userId, Long routeTableId) throws Exception {
        String sql = "update route_table_association set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId and region_id = :regionId and route_table_id = :routeTableId";
        Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
        params.put("regionId", regionId);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        params.put("routeTableId", routeTableId);
        return this.execute(sql, params) > 0;
    }

    public List<RouteTableAssociation> listByVid(Long vendorId, String regionId, Long routeTableId) throws Exception {
        String sql = "select a.* from route_table_association a  where a.is_deleted = 0 and a.vendor_id = :vendorId and a.region_id = :regionId and route_table_id = :routeTableId";
        Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
        params.put("regionId", regionId);
        params.put("routeTableId", routeTableId);
        return this.list(RouteTableAssociation.class, sql, params);
    }
}
