package com.bocloud.ims.repository.resource;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.ServerTemplateVersion;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 密钥DAO实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年10月12日
 */
@Repository("serverTemplateVersionRepository")
public class ServerTemplateVersionRepository extends BasicGenericDao<ServerTemplateVersion, Long> {

	public ServerTemplateVersionRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
		super(jdbcTemplate, npJdbcTemplate);
	}

	public List<ServerTemplateVersion> list(int page, int rows, List<Param> params, Map<String, String> sorter)
			throws Exception {
		String sql = " SELECT a.* FROM template_version a WHERE a.is_deleted = 0 ";
		sql = QueryBuilder.build(sql, new Pager(page, rows, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		return this.list(ServerTemplateVersion.class, sql, paramMap);
	}

	public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = " SELECT a.* FROM template_version a WHERE a.is_deleted = 0 ";
		sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		List<ServerTemplateVersion> serverTemplates = this.list(ServerTemplateVersion.class, sql, paramMap);
		List<SimpleBean> beans = new ArrayList<>();
		for (ServerTemplateVersion template : serverTemplates) {
			beans.add(new SimpleBean(template.getId(), template.getName()));
		}
		return beans;
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(1) from server_template a where a.is_deleted = 0";
		sql = QueryBuilder.build(sql, params, null, "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		return this.countQuery(sql, paramMap).intValue();
	}

	public Boolean remove(Long id, Long userId) throws Exception {
		String sql = "update template_version set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public ServerTemplateVersion query(Long id) throws Exception {
		String sql = " SELECT a.* FROM template_version a WHERE a.id = :id ";
		Map<String, Object> paramMap = MapTools.simpleMap("id", id);
		List<ServerTemplateVersion> list = this.list(ServerTemplateVersion.class, sql, paramMap);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public Boolean deleteByVendorId(Long vendorId, String region) throws Exception {
		String sql = " DELETE FROM template_version  WHERE vendor_id =  :vendorId and region = :region ";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("region", region);
		return this.execute(sql, params) > 0;
	}

	public List<ServerTemplateVersion> listByVendorId(Long vendorId, String region) throws Exception {
		String sql = " SELECT a.* FROM template_version a WHERE a.is_deleted = 0 and a.vendor_id = :vendorId  and a.region = :region  ";
		Map<String, Object> paramMap = MapTools.simpleMap(Common.VENDORID, vendorId);
		paramMap.put("region", region);
		return this.list(ServerTemplateVersion.class, sql, paramMap);
	}

}
