package com.bocloud.ims.repository.resource;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.VpnConn;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * vpn站点连接
 *
 * @author Administrator
 */
@Repository("vpnConnRepository")
public class VpnConnRepository extends BasicGenericDao<VpnConn, Long> {

	public VpnConnRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
		super(jdbcTemplate, npJdbcTemplate);
	}

	public List<VpnConn> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = " select * from vpn_conn v where v.is_deleted = 0 ";
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "v");
		return this.list(VpnConn.class, sql);
	}

	public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = " select * from vpn_conn v where v.is_deleted = 0 ";
		sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "v");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		List<VpnConn> vpns = this.list(VpnConn.class, sql, paramMap);
		List<SimpleBean> beans = new ArrayList<>();
		for (VpnConn vpnConn : vpns) {
			beans.add(new SimpleBean(vpnConn.getId(), vpnConn.getName()));
		}
		return beans;
	}

	public List<VpnConn> listByVendorId(Long vendorId) throws Exception {
		String sql = "select a.* from vpn_conn a where a.is_deleted = 0 and a.vendor_id =:vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		return this.list(VpnConn.class, sql, params);
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(1) from vpn_conn v where v.is_deleted = 0 ";
		sql = QueryBuilder.buildRaw(sql, params, null, "v");
		Map<String, Object> param = QueryBuilder.getParam(params);
		return this.countQuery(sql, param).intValue();
	}

	public boolean remove(Long id, Long userId) throws Exception {
		String sql = "update vpn_conn set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id ";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		params.put("gmtModify", new Date());
		params.put("menderId", userId);
		return this.execute(sql, params) > 0;
	}

	public boolean removeByVid(Long vendorId, Long userId) throws Exception {
		String sql = "update vpn_conn set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where vendor_id = :vendorId ";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("gmtModify", new Date());
		params.put("menderId", userId);
		return this.execute(sql, params) > 0;
	}

	public VpnConn query(Long id) throws Exception {
		String sql = " select * from vpn_conn v where v.is_deleted = 0 and v.id = :id ";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		List<VpnConn> list = this.list(VpnConn.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public int count(List<Param> params, Long tenantId) throws Exception {
		String sql = "select count(1) from vpn_conn v where v.is_deleted = 0 ";
		if (null != tenantId) {
			sql += " and v.tenant_id =  " + tenantId;
		}
		sql = QueryBuilder.buildRaw(sql, params, null, "v");
		Map<String, Object> param = QueryBuilder.getParam(params);
		return this.countQuery(sql, param).intValue();
	}

	public List<VpnConn> list(int page, int rows, List<Param> params, Map<String, String> sorter, Long tenantId)
			throws Exception {
		String sql = " select v.* from vpn_conn v where v.is_deleted = 0 ";
		if (null != tenantId) {
			sql += " and v.tenant_id =  " + tenantId;
		}
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "v");
		return this.list(VpnConn.class, sql);
	}

	public VpnConn queryByVpnUuid(String uuid) throws Exception {
		String sql = " select * from vpn_conn v where v.is_deleted = 0 and v.vpn_uuid = :uuid ";
		Map<String, Object> params = MapTools.simpleMap("uuid", uuid);
		List<VpnConn> list = this.list(VpnConn.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

}
