
package com.bocloud.ims.repository.resource;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.VpnPolicy;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * VpnPolicy
 *
 * @author Administrator
 */
@Repository("vpnPolicyRepository")
public class VpnPolicyRepository extends BasicGenericDao<VpnPolicy, Long> {

	public VpnPolicyRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
		super(jdbcTemplate, npJdbcTemplate);
	}

	public List<VpnPolicy> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = " select * from vpn_policy v where v.is_deleted = 0 ";
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "v");
		return this.list(VpnPolicy.class, sql);
	}

	public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = " select * from vpn_policy v where v.is_deleted = 0 ";
		sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "v");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		List<VpnPolicy> vpnPolicys = this.list(VpnPolicy.class, sql, paramMap);
		List<SimpleBean> beans = new ArrayList<>();
		for (VpnPolicy vpnPolicy : vpnPolicys) {
			beans.add(new SimpleBean(vpnPolicy.getId(), vpnPolicy.getName()));
		}
		return beans;
	}

	public List<VpnPolicy> listByVendorId(Long vendorId) throws Exception {
		String sql = "select a.* from vpn_policy a where a.is_deleted = 0 and a.vendor_id =:vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		return this.list(VpnPolicy.class, sql, params);
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(1) from vpn_policy v where v.is_deleted = 0 ";
		sql = QueryBuilder.buildRaw(sql, params, null, "v");
		Map<String, Object> param = QueryBuilder.getParam(params);
		return this.countQuery(sql, param).intValue();
	}

	public boolean remove(Long id, Long userId) throws Exception {
		String sql = "update vpn_policy set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id ";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		params.put("gmtModify", new Date());
		params.put("menderId", userId);
		return this.execute(sql, params) > 0;
	}

	public boolean removeByVid(Long vendorId, Long userId) throws Exception {
		String sql = "update vpn_policy set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where vendor_id = :vendorId ";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("gmtModify", new Date());
		params.put("menderId", userId);
		return this.execute(sql, params) > 0;
	}

	public VpnPolicy query(Long id) throws Exception {
		String sql = " select * from vpn_policy v where v.is_deleted = 0 and v.id = :id ";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		List<VpnPolicy> list = this.list(VpnPolicy.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public int count(List<Param> params, Long tenantId) throws Exception {
		String sql = "select count(1) from vpn_policy v where v.is_deleted = 0 ";
		if (null != tenantId) {
			sql += " and v.tenant_id = " + tenantId;
		}
		sql = QueryBuilder.buildRaw(sql, params, null, "v");
		Map<String, Object> param = QueryBuilder.getParam(params);
		return this.countQuery(sql, param).intValue();
	}

	public List<VpnPolicy> list(int page, int rows, List<Param> params, Map<String, String> sorter, Long tenantId)
			throws Exception {
		String sql = " select * from vpn_policy v where v.is_deleted = 0 ";
		if (null != tenantId) {
			sql += " and v.tenant_id = " + tenantId;
		}
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "v");
		return this.list(VpnPolicy.class, sql);
	}

	public List<VpnPolicy> listByTenantId(Long tenantId) throws Exception {
		// TODO Auto-generated method stub
		return null;
	}

	public List<VpnPolicy> listUsablePolicy(Long vendorId, String region, String category) throws Exception {
		String sql = " select * from vpn_policy p where p.is_deleted = 0 and p.vendor_id = :vendorId and p.category = :category and p.id not in (select c.ike_id from vpn_conn c where c.vendor_id = :vendorId and c.is_deleted = 0) ";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("category", category);
		return this.list(VpnPolicy.class, sql, params);
	}

}
