package com.bocloud.ims.repository.resource.azure.db;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.BasicDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.azure.db.DbConfiguration;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: DbConfigurationRepository
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/2/4 18:06
 */
@Repository("dbConfigurationRepository")
public class DbConfigurationRepository extends BasicDao {

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "select * from db_configuration where 1=1";
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        List<DbConfiguration> list = this.queryForList(sql, paramMap, DbConfiguration.class).stream().map(o -> (DbConfiguration) o).collect(Collectors.toList());
        List<SimpleBean> beans = Lists.newArrayList();
        for (DbConfiguration entity : list) {
            beans.add(new SimpleBean(entity.getId(), entity.getUuid()));
        }
        return beans;
    }

    public List<DbConfiguration> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "SELECT a.* " +
                "FROM db_configuration a " +
                "WHERE 1=1 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.queryForList(sql, paramMap, DbConfiguration.class).stream().map(o -> (DbConfiguration) o).collect(Collectors.toList());
    }

    public int count(List<Param> params) throws BoCloudDBException {
        String sql = "SELECT COUNT(1) " +
                "FROM db_configuration a " +
                "WHERE 1=1 ";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public DbConfiguration query(Long id) throws BoCloudDBException {
        String sql = "select a.* from db_configuration a where  a.id = :id ";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<DbConfiguration> list = this.queryForList(sql, params, DbConfiguration.class).stream().map(o -> (DbConfiguration) o).collect(Collectors.toList());
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public DbConfiguration save(DbConfiguration ip) throws BoCloudDBException {
        return (DbConfiguration) this.saveEntity(ip);
    }

    public Boolean update(DbConfiguration ip) throws BoCloudDBException {
        return this.updateEntity(ip);
    }

    public Boolean delete(Long id) throws BoCloudDBException {
        String sql = "delete from db_configuration where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        return this.update(sql, params) > 0;
    }

    public Boolean deleteByDbId(Long databaseServerId) throws BoCloudDBException {
        String sql = "delete from db_configuration where database_server_id = :databaseServerId";
        Map<String, Object> params = MapTools.simpleMap("databaseServerId", databaseServerId);
        return this.update(sql, params) > 0;
    }

    public DbConfiguration getByName(Long databaseServerId, String name) throws BoCloudDBException {
        String sql = "select a.* from db_configuration a where  a.database_server_id = :databaseServerId and a.name = :name  ";
        Map<String, Object> params = MapTools.simpleMap("databaseServerId", databaseServerId);
        params.put(Common.NAME, name);
        List<DbConfiguration> list = this.queryForList(sql, params, DbConfiguration.class).stream().map(o -> (DbConfiguration) o).collect(Collectors.toList());
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<DbConfiguration> listByDbId(Long databaseServerId) throws BoCloudDBException {
        String sql = "select * from db_configuration where database_server_id = :databaseServerId";
        Map<String, Object> params = MapTools.simpleMap("databaseServerId", databaseServerId);
        return this.queryForList(sql, params, DbConfiguration.class).stream().map(o -> (DbConfiguration) o).collect(Collectors.toList());
    }


}