package com.bocloud.ims.repository.resource.azure.db;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.BasicDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.azure.db.DbFirewallRule;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: DbFirewallRuleRepository
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/2/1 15:16
 */
@Repository("dbFirewallRuleRepository")
public class DbFirewallRuleRepository extends BasicDao {

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "select * from db_firewall_rule where 1=1";
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        List<DbFirewallRule> list = this.queryForList(sql, paramMap, DbFirewallRule.class).stream().map(o -> (DbFirewallRule) o).collect(Collectors.toList());
        List<SimpleBean> beans = Lists.newArrayList();
        for (DbFirewallRule entity : list) {
            beans.add(new SimpleBean(entity.getId(), entity.getUuid()));
        }
        return beans;
    }

    public List<DbFirewallRule> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "SELECT a.* " +
                "FROM db_firewall_rule a " +
                "WHERE 1=1 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.queryForList(sql, paramMap, DbFirewallRule.class).stream().map(o -> (DbFirewallRule) o).collect(Collectors.toList());
    }

    public int count(List<Param> params) throws BoCloudDBException {
        String sql = "SELECT COUNT(1) " +
                "FROM db_firewall_rule a " +
                "WHERE 1=1 ";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public DbFirewallRule query(Long id) throws BoCloudDBException {
        String sql = "select a.* from db_firewall_rule a where  a.id = :id ";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<DbFirewallRule> list = this.queryForList(sql, params, DbFirewallRule.class).stream().map(o -> (DbFirewallRule) o).collect(Collectors.toList());
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public DbFirewallRule save(DbFirewallRule ip) throws BoCloudDBException {
        return (DbFirewallRule) this.saveEntity(ip);
    }

    public Boolean update(DbFirewallRule ip) throws BoCloudDBException {
        return this.updateEntity(ip);
    }

    public Boolean delete(Long id) throws BoCloudDBException {
        String sql = "delete from db_firewall_rule where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        return this.update(sql, params) > 0;
    }

    public Boolean deleteByDbId(Long databaseServerId) throws BoCloudDBException {
        String sql = "delete from db_firewall_rule where database_server_id = :databaseServerId";
        Map<String, Object> params = MapTools.simpleMap("databaseServerId", databaseServerId);
        return this.update(sql, params) > 0;
    }

    public DbFirewallRule getByName(Long databaseServerId, String name) throws BoCloudDBException {
        String sql = "select a.* from db_firewall_rule a where  a.database_server_id = :databaseServerId and a.name = :name  ";
        Map<String, Object> params = MapTools.simpleMap("databaseServerId", databaseServerId);
        params.put(Common.NAME, name);
        List<DbFirewallRule> list = this.queryForList(sql, params, DbFirewallRule.class).stream().map(o -> (DbFirewallRule) o).collect(Collectors.toList());
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<DbFirewallRule> listByDbId(Long databaseServerId) throws BoCloudDBException {
        String sql = "select * from db_firewall_rule where database_server_id = :databaseServerId";
        Map<String, Object> params = MapTools.simpleMap("databaseServerId", databaseServerId);
        return this.queryForList(sql, params, DbFirewallRule.class).stream().map(o -> (DbFirewallRule) o).collect(Collectors.toList());
    }


}