package com.bocloud.ims.repository.resource.azure.db;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.BasicDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.azure.db.ServiceLevelObjective;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: ServiceLevelObjectiveRepository
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/12/22 15:01
 */
@Repository("serviceLevelObjectiveRepository")
public class ServiceLevelObjectiveRepository extends BasicDao {

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "select * from service_level_objective where 1=1";
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        List<ServiceLevelObjective> list = this.queryForList(sql, paramMap, ServiceLevelObjective.class).stream().map(o -> (ServiceLevelObjective) o).collect(Collectors.toList());
        List<SimpleBean> beans = Lists.newArrayList();
        for (ServiceLevelObjective entity : list) {
            beans.add(new SimpleBean(entity.getId(), entity.getUuid()));
        }
        return beans;
    }

    public List<ServiceLevelObjective> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "SELECT a.* " +
                "FROM service_level_objective a " +
                "WHERE 1=1 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.queryForList(sql, paramMap, ServiceLevelObjective.class).stream().map(o -> (ServiceLevelObjective) o).collect(Collectors.toList());
    }

    public int count(List<Param> params) throws BoCloudDBException {
        String sql = "SELECT COUNT(1) " +
                "FROM service_level_objective a " +
                "WHERE 1=1 ";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public ServiceLevelObjective query(Long id) throws BoCloudDBException {
        String sql = "select a.* from service_level_objective a where  a.id = :id ";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<ServiceLevelObjective> list = this.queryForList(sql, params, ServiceLevelObjective.class).stream().map(o -> (ServiceLevelObjective) o).collect(Collectors.toList());
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public ServiceLevelObjective save(ServiceLevelObjective ip) throws BoCloudDBException {
        return (ServiceLevelObjective) this.saveEntity(ip);
    }

    public Boolean update(ServiceLevelObjective ip) throws BoCloudDBException {
        return this.updateEntity(ip);
    }

    public Boolean delete(Long id) throws BoCloudDBException {
        String sql = "delete from service_level_objective where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        return this.update(sql, params) > 0;
    }

    public List<ServiceLevelObjective> listByVid(Long vendorId) throws BoCloudDBException {
        String sql = "select * from service_level_objective where vendor_id = :vendorId";
        Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
        return this.queryForList(sql, params, ServiceLevelObjective.class).stream().map(o -> (ServiceLevelObjective) o).collect(Collectors.toList());
    }


    public List<ServiceLevelObjective> listByPid(Long performanceTierId) throws BoCloudDBException {
        String sql = "select * from service_level_objective where performance_tier_id = :performanceTierId";
        Map<String, Object> params = MapTools.simpleMap("performanceTierId", performanceTierId);
        return this.queryForList(sql, params, ServiceLevelObjective.class).stream().map(o -> (ServiceLevelObjective) o).collect(Collectors.toList());
    }

    public Boolean removeByPid(Long performanceTierId) throws BoCloudDBException {
        String sql = "delete from service_level_objective where performance_tier_id = :performanceTierId";
        Map<String, Object> params = MapTools.simpleMap("performanceTierId", performanceTierId);
        return this.update(sql, params) > 0;
    }

    public List<ServiceLevelObjective> listByVid(Long vendorId, String regionId) throws BoCloudDBException {
        String sql = "select * from service_level_objective where vendor_id = :vendorId and region_id = :regionId";
        Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
        params.put(Common.REGIONID, regionId);
        return this.queryForList(sql, params, ServiceLevelObjective.class).stream().map(o -> (ServiceLevelObjective) o).collect(Collectors.toList());
    }

}