package com.bocloud.ims.repository.resource.azure.redis;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.BasicDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.azure.redis.CloudRedisFirewallRule;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: CloudRedisFirewallRuleRepository
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/3/1 14:37
 */
@Repository
public class CloudRedisFirewallRuleRepository extends BasicDao {

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "select * from cloud_redis_firewall_rule where 1=1";
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        List<CloudRedisFirewallRule> list = this.queryForList(sql, paramMap, CloudRedisFirewallRule.class).stream().map(o -> (CloudRedisFirewallRule) o).collect(Collectors.toList());
        List<SimpleBean> beans = Lists.newArrayList();
        for (CloudRedisFirewallRule entity : list) {
            beans.add(new SimpleBean(entity.getId(), entity.getUuid()));
        }
        return beans;
    }

    public List<CloudRedisFirewallRule> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "SELECT a.* " +
                "FROM cloud_redis_firewall_rule a " +
                "WHERE 1=1 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.queryForList(sql, paramMap, CloudRedisFirewallRule.class).stream().map(o -> (CloudRedisFirewallRule) o).collect(Collectors.toList());
    }

    public int count(List<Param> params) throws BoCloudDBException {
        String sql = "SELECT COUNT(1) " +
                "FROM cloud_redis_firewall_rule a " +
                "WHERE 1=1 ";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public CloudRedisFirewallRule query(Long id) throws BoCloudDBException {
        String sql = "select a.* from cloud_redis_firewall_rule a where  a.id = :id ";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<CloudRedisFirewallRule> list = this.queryForList(sql, params, CloudRedisFirewallRule.class).stream().map(o -> (CloudRedisFirewallRule) o).collect(Collectors.toList());
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public CloudRedisFirewallRule save(CloudRedisFirewallRule ip) throws BoCloudDBException {
        return (CloudRedisFirewallRule) this.saveEntity(ip);
    }

    public Boolean update(CloudRedisFirewallRule ip) throws BoCloudDBException {
        return this.updateEntity(ip);
    }

    public Boolean delete(Long id) throws BoCloudDBException {
        String sql = "delete from cloud_redis_firewall_rule where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        return this.update(sql, params) > 0;
    }

    public Boolean deleteByRedisId(Long redisId) throws BoCloudDBException {
        String sql = "delete from cloud_redis_firewall_rule where redis_id = :redisId";
        Map<String, Object> params = MapTools.simpleMap("redisId", redisId);
        return this.update(sql, params) > 0;
    }

    public CloudRedisFirewallRule getByName(Long redisId, String name) throws BoCloudDBException {
        String sql = "select a.* from cloud_redis_firewall_rule a where  a.redis_id = :redisId and a.name = :name  ";
        Map<String, Object> params = MapTools.simpleMap("redisId", redisId);
        params.put(Common.NAME, name);
        List<CloudRedisFirewallRule> list = this.queryForList(sql, params, CloudRedisFirewallRule.class).stream().map(o -> (CloudRedisFirewallRule) o).collect(Collectors.toList());
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    public List<CloudRedisFirewallRule> listByRedisId(Long redisId) throws BoCloudDBException {
        String sql = "select * from cloud_redis_firewall_rule where redis_id = :redisId";
        Map<String, Object> params = MapTools.simpleMap("redisId", redisId);
        return this.queryForList(sql, params, CloudRedisFirewallRule.class).stream().map(o -> (CloudRedisFirewallRule) o).collect(Collectors.toList());
    }

}