package com.bocloud.ims.repository.resource.f5;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.f5.BiIRules;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Repository("f5BiIRulesRepository")
public class F5BiIRulesRepository extends BasicGenericDao<BiIRules, Long> {

    public F5BiIRulesRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<BiIRules> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.*,b.name as partition_name from bi_rule a LEFT JOIN bi_partition b ON a.partition_id = b.id where a.is_deleted = 0 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(BiIRules.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select b.id,b.name from bi_rule b where b.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "b");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        List<BiIRules> rules = this.list(BiIRules.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<SimpleBean>();
        for (BiIRules rule : rules) {
            beans.add(new SimpleBean(rule.getId(), rule.getName()));
        }
        return beans;
    }

    public List<BiIRules> list(Long vendorId) throws Exception {
        String sql = "select * from bi_rule a where a.is_deleted = 0 and a.vendor_id = :vendorId ";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        return this.list(BiIRules.class, sql, params);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from bi_rule a where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public boolean remove(Long id, Long userId) throws Exception {
        String sql = "update bi_rule set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public BiIRules query(Long id) throws Exception {
        String sql = "select * from bi_rule where id =:id and is_deleted = 0";
        Map<String, Object> param = MapTools.simpleMap(Common.ID, id);
        List<BiIRules> iRules = this.list(BiIRules.class, sql, param);
        if (iRules.isEmpty()) {
            return null;
        }
        return iRules.get(0);
    }

    public boolean removeByVendor(Long vendorId, Long userId) throws Exception {
        String sql = "update bi_rule set is_deleted = 1, gmt_modify = :gmtModify ,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public boolean existName(String name, Long vendorId, Long partitionId) throws Exception {
        String sql = "select a.* from bi_rule a where a.is_deleted = 0 and a.name = :name and a.vendor_id = :vendorId and partition_id = :partitionId ";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        paramMap.put("vendorId", vendorId);
        paramMap.put("partitionId", partitionId);
        List<BiIRules> biIRules = this.list(BiIRules.class, sql, paramMap);
        if (biIRules.isEmpty()) {
            return false;
        }
        return true;
    }

    public BiIRules queryByName(String name, Long vendorId, Long partitionId) throws Exception {
        String sql = "select * from bi_rule where name =:name and is_deleted = 0 and vendor_id = :vendorId and partition_id = :partitionId ";
        Map<String, Object> param = MapTools.simpleMap(Common.NAME, name);
        param.put("vendorId", vendorId);
        param.put("partitionId", partitionId);
        List<BiIRules> iRules = this.list(BiIRules.class, sql, param);
        if (iRules.isEmpty()) {
            return null;
        }
        return iRules.get(0);
    }

    public List<BiIRules> query(List<Long> ruleIds) throws Exception {
        String sql = "select * from bi_rule a where a.is_deleted = 0 and id in(:ruleIds)";
        Map<String, Object> params = MapTools.simpleMap("ruleIds", ruleIds);
        return this.list(BiIRules.class, sql, params);
    }

}
