package com.bocloud.ims.repository.resource.f5;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.f5.BiNode;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author kyw
 */
@Repository("f5BiNodeRepository")
public class F5BiNodeRepository extends BasicGenericDao<BiNode, Long> {

    public F5BiNodeRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<BiNode> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.*,bp.name as partition_name from bi_node a left join bi_partition bp on a.partition_id = bp.id where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(BiNode.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select b.id,b.name from bi_node b where b.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "b");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        List<BiNode> listBiNode = this.list(BiNode.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<SimpleBean>();
        for (BiNode biNode : listBiNode) {
            beans.add(new SimpleBean(biNode.getId(), biNode.getName()));
        }
        return beans;
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from bi_node a where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public boolean remove(Long id, Long userId) throws Exception {
        String sql = "update bi_node set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public BiNode query(Long id) throws Exception {
        String sql = "select a.*,bp.name AS partition_name from bi_node a LEFT JOIN bi_partition bp ON a.partition_id = bp.id where a.id =:id and a.is_deleted = 0";
        Map<String, Object> param = MapTools.simpleMap(Common.ID, id);
        List<BiNode> listBiNode = this.list(BiNode.class, sql, param);
        if (listBiNode.isEmpty()) {
            return null;
        }
        return listBiNode.get(0);
    }

    public boolean removeByVendor(Long vendorId, Long userId) throws Exception {
        String sql = "update bi_node set is_deleted = 1, gmt_modify = :gmtModify ,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public boolean existName(String name, Long vendorId, Long partitionId) throws Exception {
        String sql = "select a.* from bi_node a where a.is_deleted = 0 and a.name = :name and a.vendor_id = :vendorId and partition_id = :partitionId ";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        paramMap.put("vendorId", vendorId);
        paramMap.put("partitionId", partitionId);
        List<BiNode> listBiNode = this.list(BiNode.class, sql, paramMap);
        if (listBiNode.isEmpty()) {
            return false;
        }
        return true;
    }

    public List<BiNode> list(Long vendorId) throws Exception {
        String sql = "select * from bi_node a where a.is_deleted = 0 and a.vendor_id = :vendorId ";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        return this.list(BiNode.class, sql, params);
    }

    public BiNode queryByAddress(String address, Long vendorId, Long partitionId) throws Exception {
        String sql = "select * from bi_node where address =:address and is_deleted = 0 and vendor_id = :vendorId and partition_id = :partitionId ";
        Map<String, Object> param = MapTools.simpleMap("address", address);
        param.put("vendorId", vendorId);
        param.put("partitionId", partitionId);
        List<BiNode> listBiNode = this.list(BiNode.class, sql, param);
        if (listBiNode.isEmpty()) {
            return null;
        }
        return listBiNode.get(0);
    }

}
