package com.bocloud.ims.repository.resource.f5;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.f5.BiPartition;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Repository("f5BiPartitionRepository")
public class F5BiPartitionRepository extends BasicGenericDao<BiPartition, Long> {

    public F5BiPartitionRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<BiPartition> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select * from bi_partition a where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(BiPartition.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select b.id,b.name from bi_partition b where b.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "b");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        List<BiPartition> listPartition = this.list(BiPartition.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<SimpleBean>();
        for (BiPartition partition : listPartition) {
            beans.add(new SimpleBean(partition.getId(), partition.getName()));
        }
        return beans;
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from bi_partition a where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public boolean remove(Long id, Long userId) throws Exception {
        String sql = "update bi_partition set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public boolean existName(String name, Long vendorId) throws Exception {
        String sql = "select a.* from bi_partition a where a.is_deleted = 0 and a.name = :name and a.vendor_id = :vendorId ";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        paramMap.put("vendorId", vendorId);
        List<BiPartition> listBiPartition = this.list(BiPartition.class, sql, paramMap);
        if (listBiPartition.isEmpty()) {
            return false;
        }
        return true;
    }

    public BiPartition query(Long id) throws Exception {
        String sql = "select * from bi_partition where id =:id and is_deleted = 0";
        Map<String, Object> param = MapTools.simpleMap(Common.ID, id);
        List<BiPartition> listBiPartition = this.list(BiPartition.class, sql, param);
        if (listBiPartition.isEmpty()) {
            return null;
        }
        return listBiPartition.get(0);
    }

    public boolean removeByVendor(Long vendorId, Long userId) throws Exception {
        String sql = "update bi_partition set is_deleted = 1, gmt_modify = :gmtModify ,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        params.put("menderId", userId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public List<BiPartition> list(Long vendorId) throws Exception {
        String sql = "select * from bi_partition a where a.is_deleted = 0 and a.vendor_id = :vendorId ";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        return this.list(BiPartition.class, sql, params);
    }

    public BiPartition queryByName(String name, Long vendorId) throws Exception {
        String sql = "select * from bi_partition where name =:name and is_deleted = 0 and vendor_id = :vendorId ";
        Map<String, Object> param = MapTools.simpleMap(Common.NAME, name);
        param.put("vendorId", vendorId);
        List<BiPartition> biPartitions = this.list(BiPartition.class, sql, param);
        if (biPartitions.isEmpty()) {
            return null;
        }
        return biPartitions.get(0);
    }

}
