package com.bocloud.ims.repository.resource.hmc;

import com.bocloud.ims.entity.common.MapBuilder;
import com.bocloud.ims.entity.resource.hmc.HmdServerVirtualBridge;
import org.springframework.beans.factory.annotation.Lookup;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;

@Repository
public abstract class HmdServerVirtualBridgeRepository extends HmdGenericDao<HmdServerVirtualBridge> {

    @Lookup protected abstract HmdServerRepository serverRepository();

    public HmdServerVirtualBridgeRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    @Override
    public void extra(HmdServerVirtualBridge bridge) {
        Optional.of(bridge).map(HmdServerVirtualBridge::getServerId).map(this.serverRepository()::getById).ifPresent(bridge::setServer);
    }

    public List<HmdServerVirtualBridge> listByServerId(long serverId) {
        return this.search(Collections.singletonMap("a.server_id", serverId));
    }

    public HmdServerVirtualBridge getById(long id) {
        return this.search(Collections.singletonMap("a.id", id)).stream().findFirst().orElse(null);
    }

    public HmdServerVirtualBridge getByUuid(String uuid) {
        return this.search(Collections.singletonMap("a.uuid", uuid)).stream().findFirst().orElse(null);
    }

    public List<HmdServerVirtualBridge> search(long serverId, String bridgeName) {
        Map<String, Object> map = MapBuilder.init()
            .put("a.server_id", serverId)
            .put("a.name", bridgeName)
            .get();
        return this.search(map);
    }

}
