package com.bocloud.ims.repository.resource.horizon;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.ims.entity.resource.horizon.VmwareHorizonUserOrGroup;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * vmware horizon用户，组
 * 
 * @author HP
 *
 */
@Repository("vmwareHorizonUserOrGroupRepository")
public class VmwareHorizonUserOrGroupRepository extends BasicGenericDao<VmwareHorizonUserOrGroup, Long> {

	public VmwareHorizonUserOrGroupRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
		super(jdbcTemplate, npJdbcTemplate);
	}

	public List<VmwareHorizonUserOrGroup> list(int page, int rows, List<Param> params, Map<String, String> sorter)
			throws Exception {
		String sql = "select a.*,v.name vendor_name from vmware_horizon_user_or_group a left join cloud_vendor v on v.id = a.vendor_id where a.is_deleted = 0 ";
		sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		return this.list(VmwareHorizonUserOrGroup.class, sql, paramMap);
	}

	public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = "select a.id,a.name from vmware_horizon_user_or_group a where a.is_deleted = 0";
		sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		List<VmwareHorizonUserOrGroup> groups = this.list(VmwareHorizonUserOrGroup.class, sql, paramMap);
		List<SimpleBean> beans = new ArrayList<>();
		for (VmwareHorizonUserOrGroup group : groups) {
			beans.add(new SimpleBean(group.getId(), group.getName()));
		}
		return beans;
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(a.id) from vmware_horizon_user_or_group a where a.is_deleted = 0";
		sql = QueryBuilder.buildRaw(sql, params, null, "a");
		Map<String, Object> paramMap = QueryBuilder.getParam(params);
		return this.countQuery(sql, paramMap).intValue();
	}

	public VmwareHorizonUserOrGroup query(Long id) throws Exception {
		String sql = "select a.*,v.name vendor_name from vmware_horizon_user_or_group a left join cloud_vendor v on v.id = a.vendor_id where a.is_deleted = 0 and a.id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		List<VmwareHorizonUserOrGroup> list = this.list(VmwareHorizonUserOrGroup.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public List<VmwareHorizonUserOrGroup> listByVendorId(Long vendorId) {
		String sql = "select a.*,v.name vendor_name from vmware_horizon_user_or_group a left join cloud_vendor v on v.id = a.vendor_id where a.is_deleted = 0 and a.vendor_id = :vendorId";
		Map<String, Object> paramMap = MapTools.simpleMap(Common.VENDORID, vendorId);
		List<VmwareHorizonUserOrGroup> list = this.list(VmwareHorizonUserOrGroup.class, sql, paramMap);
		if (list.isEmpty()) {
			return new ArrayList<>();
		}
		return list;
	}

	public boolean removeByVendorId(Long vendorId, Long menderId) {
		String sql = "update vmware_horizon_user_or_group set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("menderId", menderId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public boolean remove(Long id, Long menderId) {
		String sql = "update vmware_horizon_user_or_group set is_deleted = true,gmt_modify = :gmtModify,mender_id = :menderId where is_deleted = 0 and id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		params.put("menderId", menderId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public boolean existByCondition(String column, String value, Long vendorId) throws Exception {
		String sql = "select * from vmware_horizon_user_or_group where " + column
				+ " = :value and vendor_id = :vendorId and is_deleted = 0";
		Map<String, Object> params = MapTools.simpleMap("value", value);
		params.put(Common.VENDORID, vendorId);
		List<VmwareHorizonUserOrGroup> list = this.list(VmwareHorizonUserOrGroup.class, sql, params);
		if (list.isEmpty()) {
			return false;
		}
		return true;
	}

}
