package com.bocloud.ims.service.builder;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.driver.compute.ServerDriver;
import com.bocloud.cmp.driver.storage.VolumeDriver;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.entity.enums.ResourceCategory;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.utils.IPAddress;
import com.bocloud.common.enums.ResourceCatalog;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.IDFactory;
import com.bocloud.common.utils.ListTool;
import com.bocloud.ims.entity.VolumeType;
import com.bocloud.ims.entity.resource.*;
import com.bocloud.ims.entity.resource.Ip.IpStatus;
import com.bocloud.ims.entity.resource.manageone.ManageOneAccount;
import com.bocloud.ims.interfaces.manageone.ManageOneAccountService;
import com.bocloud.ims.repository.resource.*;
import com.bocloud.ims.service.utils.PwdRandom;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 华为manage one云主机创建类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/9/26
 */
public class ManageOneServerBuilder implements ServerBuilder {

    private static Logger logger = LoggerFactory.getLogger(ManageOneServerBuilder.class);
    private CloudServerRepository cloudServerRepository;
    private NetworkRepository networkRepository;
    private SubnetRepository subnetRepository;
    private ImageRepository imageRepository;
    private FlavorRepository flavorRepository;
    private VolumeRepository volumeRepository;
    private PortRepository portRepository;
    private PortFixedIpRepository portFixedIpRepository;
    private IpRepository ipRepository;
    private IpPoolRepository ipPoolRepository;
    private ResourceRepository resourceRepository;
    private ResourceEventRepository resourceEventRepository;
    private SecurityGroupRepository securityGroupRepository;
    private ServerSgroupRepository serverSgroupRepository;
    private KeypairRepository keypairRepository;
    private RegionRepository regionRepository;
    private ServerDriver serverDriver;
    private VolumeDriver volumeDriver;
    private RequestContext context;
    private CloudVendor vendor;
    private ServerModel model;
    private RouterRepository routerRepository;
    private ManageOneAccountService accountService;
    private ServerRepository serverRepository;
    private VolumeTypeRepository volumeTypeRepository;

    public ManageOneServerBuilder(CloudServerRepository cloudServerRepository, NetworkRepository networkRepository,
                                  SubnetRepository subnetRepository, ImageRepository imageRepository, FlavorRepository flavorRepository,
                                  VolumeRepository volumeRepository, PortRepository portRepository,
                                  PortFixedIpRepository portFixedIpRepository, IpRepository ipRepository, IpPoolRepository ipPoolRepository,
                                  ResourceRepository resourceRepository, ResourceEventRepository resourceEventRepository,
                                  SecurityGroupRepository securityGroupRepository, ServerSgroupRepository serverSgroupRepository,
                                  KeypairRepository keypairRepository,
                                  RegionRepository regionRepository, ServerDriver serverDriver, VolumeDriver volumeDriver,
                                  RequestContext context, CloudVendor vendor, RouterRepository routerRepository,
                                  ManageOneAccountService accountService, ServerRepository serverRepository, VolumeTypeRepository volumeTypeRepository) {
        this.cloudServerRepository = cloudServerRepository;
        this.networkRepository = networkRepository;
        this.subnetRepository = subnetRepository;
        this.imageRepository = imageRepository;
        this.flavorRepository = flavorRepository;
        this.volumeRepository = volumeRepository;
        this.portRepository = portRepository;
        this.portFixedIpRepository = portFixedIpRepository;
        this.ipRepository = ipRepository;
        this.ipPoolRepository = ipPoolRepository;
        this.resourceRepository = resourceRepository;
        this.resourceEventRepository = resourceEventRepository;
        this.securityGroupRepository = securityGroupRepository;
        this.serverSgroupRepository = serverSgroupRepository;
        this.keypairRepository = keypairRepository;
        this.regionRepository = regionRepository;
        this.serverDriver = serverDriver;
        this.volumeDriver = volumeDriver;
        this.context = context;
        this.vendor = vendor;
        this.routerRepository = routerRepository;
        this.accountService = accountService;
        this.serverRepository = serverRepository;
        this.volumeTypeRepository = volumeTypeRepository;
    }

    @Override
    public BocloudResult<ServerModel> prepare(JSONObject object) throws Exception {
        try {
            String networks = object.getString("networks");// 网络
            String groups = object.getString("groups");// 安全组
            Long imageId = object.getLong("imageId");// 镜像
            Long flovorId = object.getLong("flavorId");// 规格
            String password = object.getString("password");// 密码
            String keypairName = object.getString("keypairName");// 密钥
            String name = object.getString("name");// 云主机名称
            String rootVolumeType = object.getString("rootVolumeType");// 云主机名称
            // 2019年4月8日10:56:21 天津港 前端传递参数为字符串 区域名称
            String regionName = object.getString("region");
            Long vendorId = object.getLong("vendorId");
            Integer rootSize = object.getInteger("rootSize");// 根磁盘大小
            model = new ServerModel();
            model.setRootSize(rootSize);
            model.setDisk(rootSize);
            Region region = regionRepository.getByName(regionName, vendorId);
            if (null == region) {
                return new BocloudResult<>(false, model, "所属域不存在");
            }
            model.setZone(region.getName());
            
            // 云主机标签
            if (!StringUtils.isEmpty(object.getString("labelIds"))) {
                List<Long> labelIds = JSONArray.parseArray(object.getString("labelIds"), Long.class);
                model.setLabelIds(labelIds);
            }
            String account = null;
            // 如果未设置密码则随机生成并保存
            if (StringUtils.isEmpty(keypairName) && StringUtils.isEmpty(password)) {
                password = randomStr(12);
            }
            if (!StringUtils.isEmpty(keypairName)) {
                // 查询密钥是否存在
                Keypair keypair = keypairRepository.getByName(keypairName, vendor.getId());
                if (null == keypair) {
                    return new BocloudResult<>(false, model, "密钥不存在");
                }
            }
            model.setKeypairName(keypairName);
            if (null != imageId && 0L != imageId) {
                Image image = imageRepository.query(imageId);
                if (null == image) {
                    return new BocloudResult<>(false, model, "镜像不存在");
                }
                model.setOsName(image.getOsName());
                model.setOsCategory(image.getOsType());
                model.setBootFromVolume(true);
                model.setImageUuid(image.getImageUuid());
            }
            // 规格
            Flavor flavor = flavorRepository.query(flovorId);
            if (null == flavor) {
                return new BocloudResult<>(false, model, "规格不存在");
            }
            model.setCpu(flavor.getCpu());
            model.setMemory(flavor.getMemory());
            List<NetworkBean> nets = JSONArray.parseArray(networks, NetworkBean.class);
            if (!ListTool.isEmpty(nets)) {
                // 校验网络
                for (NetworkBean bean : nets) {
                    Long routerId = bean.getRouterId();
                    Router router = routerRepository.query(routerId);
                    if (null == router) {
                        return new BocloudResult<>(false, model, "虚拟私有云数据异常！");
                    }
                    model.setRouterUuid(router.getRouterUuid());
                    // 网络下子网IP地址
                    List<AddressBean> addresses = bean.getAddresses();
                    if (!ListTool.isEmpty(addresses)) {
                        for (AddressBean address : addresses) {
                            Subnet subnet = subnetRepository.query(address.getSubnetId());
                            if (null == subnet) {
                                return new BocloudResult<>(false, model, "子网不存在");
                            }
                            Network network = networkRepository.getByNetId(subnet.getNetworkUuid());
                            if (null == network) {
                                return new BocloudResult<>(false, model, "当前网络异常，创建失败！");
                            }
                            bean.setNetworkUuid(subnet.getNetworkUuid());
                            address.setNetworkUuid(subnet.getNetworkUuid());
                            address.setSubnetUuid(subnet.getSubnetUuid());
                            address.setSubnetName(subnet.getName());
                            address.setNetworkId(network.getId());
                            String addr = address.getAddress();
                            if (StringUtils.isEmpty(addr)) {
                                continue;
                            }
                            // 判断IP是否存在于指定网络的IP池中
//                            Ip ip = ipRepository.checkExist(addr, subnet.getId(), vendor.getId());
//                            if (null != ip) {
//                                return new BocloudResult<>(false, model, "指定的IP地址【" + addr + "】已被使用");
//                            }
                            // 判断ip是否存在子网的范围内
                            IpPool ipPool = ipPoolRepository.queryBySubnet(subnet.getId(), subnet.getVendorId());
                            if (null == ipPool) {
                                ipPool = new IpPool(subnet.getGateway(), subnet.getCidr(), subnet.getIpPools(),
                                        subnet.getDnsNames(), subnet.getId(), network.getId(), subnet.getVendorId());
                                ipPool.setCreatorId(context.getTarget());
                                ipPool.setMenderId(context.getTarget());
                                ipPool.setVersion(IPAddress.getVersion(subnet.getGateway()));
                                ipPoolRepository.save(ipPool);
                            }
                            List<String> pools = JSONArray.parseArray(ipPool.getPools(), String.class);
                            if (ListTool.isEmpty(pools)) {
                                return new BocloudResult<>(false, model, "该子网IP地址池分配有误");
                            }
                            IPAddress ipAddress = IPAddress.getIPAddress(addr);
                            boolean flag = ipAddress.isInPools(addr, pools);
                            if (!flag) {
                                return new BocloudResult<>(false, model, "该IP不在该子网的网段内");
                            }
//                            ip = new Ip(addr, ipAddress.getMask(subnet.getCidr()), subnet.getGateway(),
//                                    subnet.getDnsNames(), vendor.getId(), IDFactory.instance().uuid(), ipPool.getId(),
//                                    IpStatus.used);
//                            ip.setCatalog("PORT");
//                            ip.setNetworkId(subnet.getNetworkId());
//                            ip.setSubnetId(subnet.getId());
//                            ipRepository.save(ip);
                            bean.setFixIp(addr);
                        }
                    }
                }
            }
            if(StringUtils.isEmpty(rootVolumeType)) {
            	List<VolumeType> volumeTypes = volumeTypeRepository.listByZone(vendorId, model.getZone());
            	if(!ListTool.isEmpty(volumeTypes)) {
            		rootVolumeType = volumeTypes.get(0).getVolumeTypeUuid();
            	}
            }
            model.setRootVolumeType(rootVolumeType);
            List<String> securityGroups = Lists.newArrayList();
            if (!StringUtils.isEmpty(groups)) {
                securityGroups = JSONArray.parseArray(groups, String.class);
            }
            model.setResourceLabel(object.getString("resourceLabel"));
            model.prepareData(name, account, password, flavor.getFlavorUuid(), nets, securityGroups);
        } catch (Exception e) {
            logger.error("Get error Message:", e);
            return new BocloudResult<>(false, model, "fail");
        }
        return new BocloudResult<>(true, model, "success");
    }

    /**
     * 回收IP
     *
     * @param networkBeans
     * @param vendorId
     * @return
     */
    private void recycleIp(List<NetworkBean> networkBeans, Long vendorId) {
        try {
            if (!ListTool.isEmpty(networkBeans)) {
                for (NetworkBean networkBean : networkBeans) {
                    List<AddressBean> addresses = networkBean.getAddresses();
                    if (ListTool.isEmpty(addresses)) {
                        continue;
                    }
                    for (AddressBean address : addresses) {
                        ipRepository.delete(address.getAddress(), address.getSubnetId(), vendor.getId());
                    }
                }
            }
        } catch (Exception e) {
            logger.error("recycle ip error:", e);
        }
    }

    @Override
    public BocloudResult<CloudServer> builder(CloudServer cloudServer) throws Exception {
        // 创建云主机
        BocloudResult result = new BocloudResult(false, "云主机创建失败");
        try {
            // 操作记录
            Long userId = context.getTarget();
            String handler = "云主机创建";
            ResourceEvent event = null;
            Resource resource = null;
            // 创建云主机
            result = serverDriver.create(vendor.getUuid(), null, model);
            if (result.isSuccess()) {
                event = new ResourceEvent(cloudServer.getId(), handler, "成功", userId, new Date(), "VM", vendor.getId(),
                        context.getCatalog());
                String content = JSONObject.toJSONString(result.getData());
                CloudServer server = JSONObject.parseObject(content, CloudServer.class);
                List<Volume> diskDevices = server.getDiskDevices();
                for (Volume volume : diskDevices) {
					BocloudResult volumeResult = volumeDriver.detail(volume.getVolumeUuid(), vendor.getUuid(), null);
					if (volumeResult.isSuccess()) {
						volume = JSONObject.parseObject(JSONObject.toJSONString(volumeResult.getData()), Volume.class);
					}
                    List<ServerVolumeModel> models = volume.getModels();
                    Map<String, ServerVolumeModel> serverVolumeMap = Maps.newHashMap();
                    if (!ListTool.isEmpty(models)) {
                        for (ServerVolumeModel model : models) {
                            serverVolumeMap.put(model.getInstanceId(), model);
                        }
                    }
                    ServerVolumeModel model = serverVolumeMap.get(server.getInstanceId());
                    String device = null;
                    if (null != model) {
                        device = model.getDevice();
                    }
                    Volume existVolume = volumeRepository.getByVolumeUuid(volume.getVolumeUuid());
                    Long volumeId = null;
                    Integer volumeSize = 0;
                    Region region = regionRepository.getByName(volume.getZone(), vendor.getId());
                    if (null != region) {
                        volume.setRegion(region.getName());
                    }
                    if (null == existVolume) {
                        volume.setCreatorId(userId);
                        volume.setGmtCreate(new Date());
                        volume.setVendorId(vendor.getId());
                        volume.setDevice(device);
                        volume.setServerId(cloudServer.getId());
                        volume.setTenantId(context.getTenant());
                        volume.setProjectId(context.getProject());
                        volumeRepository.save(volume);
                        volumeId = volume.getId();
                        volumeSize = volume.getSize();
                    } else {
                        // 更新
                        existVolume.setStatus("IN_USE");
                        existVolume.setDevice(device);
                        existVolume.setGmtModify(new Date());
                        existVolume.setMenderId(context.getTarget());
                        existVolume.setServerId(cloudServer.getId());
                        existVolume.setTenantId(context.getTenant());
                        existVolume.setProjectId(context.getProject());
                        volumeRepository.update(existVolume);
                        volumeId = existVolume.getId();
                        volumeSize = existVolume.getSize();
                    }
                    if (null != volume.getBootVolume() && volume.getBootVolume()) {
                        cloudServer.setDisk(volumeSize);
                        cloudServer.setVolumeId(volumeId);
                    }
                }
                String hostName = server.getHostName();
                Server host = serverRepository.getByHostName(hostName, vendor.getId());
                if (null != host) {
                    cloudServer.setCpuModel(host.getCpuModel());
                    cloudServer.setModelName(host.getModelName());
                    cloudServer.setHostId(host.getId());
                }
                cloudServer.setHostName(server.getHostName());
                String imageUuid = server.getImageUuid();
                Image image = imageRepository.getByImageUuid(imageUuid);
                if (null != image) {
                    cloudServer.setOsCategory(image.getOsCategory());
                    cloudServer.setOsName(image.getOsName());
                    cloudServer.setOsVersion(image.getOsVersion());
                }
                cloudServer.setRegion(server.getZone());
                cloudServer.setRegionId(server.getZone());
                cloudServer.setImageUuid(server.getImageUuid());
                cloudServer.setAccount(model.getAccount());
                cloudServer.setGroups(server.getGroups());
                cloudServer.setInstanceId(server.getInstanceId());
                cloudServer.setUuid(server.getUuid());
                cloudServer.setZone(server.getZone());
                String flavorUuid = server.getFlavorUuid();
                if (!StringUtils.isEmpty(flavorUuid)) {
                    Flavor flavor = flavorRepository.getByFlavorUuid(flavorUuid);
                    if (null != flavor) {
                        cloudServer.setCpu(flavor.getCpu());
                        cloudServer.setMemory(flavor.getMemory());
                    }
                }
                cloudServer.setRouterUuid(model.getRouterUuid());
                cloudServer.setKeypairName(server.getKeypairName());
                cloudServer.setFlavorUuid(flavorUuid);
                cloudServer.setGmtVmCreate(server.getGmtVmCreate());
                cloudServer.setManagerIp(server.getManagerIp());
                cloudServer.setRegion(server.getRegion());
                cloudServer.setTenantUuid(server.getTenantUuid());
                String state = server.getStatus();
                logger.info("Start to update vm status...");
                updateStatus(cloudServer, state);
                logger.info("End to update vm status...");
                if (state.equalsIgnoreCase(BocloudStatus.Server.EXCEPTION.name())
                        || state.equalsIgnoreCase(BocloudStatus.Server.SUSPENDED.name())) {
                    event = new ResourceEvent(cloudServer.getId(), handler, "失败", userId, new Date(), "VM",
                            vendor.getId(), context.getCatalog());
                }
                logger.info("Start to sync vm port...");
                List<PortModel> ports = server.getPorts();
                // 保存端口
                Map<String, List<PortModel>> portMap = Maps.newHashMap();
                Map<String, NetworkBean> networkMap = Maps.newHashMap();
                Map<String, AddressBean> subnetMap = Maps.newHashMap();
                for (PortModel portModel : ports) {
                    Port port = new Port();
                    BeanUtils.copyProperties(portModel, port);
                    Network network = networkRepository.getByNetId(portModel.getNetworkUuid());
                    NetworkBean networkBean = new NetworkBean(network.getId(), network.getName(),
                            network.getNetworkUuid());
                    networkMap.put(portModel.getNetworkUuid(), networkBean);
                    port.preSave(userId, userId, vendor.getId(), network.getId());
                    port.setStatus(BocloudStatus.Network.RUNNING.name());
                    port.setDeviceOwner("compute:None");
                    port.setDeviceUuid(server.getInstanceId());
                    portRepository.save(port);
                    // 保存fixedip
                    List<PortFixedIpModel> fixedIpModels = portModel.getPortFixedIps();
                    if (!ListTool.isEmpty(fixedIpModels)) {
                        for (PortFixedIpModel fixedIpModel : fixedIpModels) {
                            PortFixedIp portFixedIp = new PortFixedIp();
                            BeanUtils.copyProperties(fixedIpModel, portFixedIp);
                            portFixedIp.setPortId(port.getId());
                            portFixedIp.setPortUuid(portModel.getPortUuid());
                            Subnet subnet = subnetRepository.getBySubId(fixedIpModel.getSubnetUuid());
                            if (null != subnet) {
                                portFixedIp.setSubnetId(subnet.getId());
                                AddressBean addressBean = new AddressBean(subnet.getId(), subnet.getName(),
                                        network.getId(), subnet.getSubnetUuid(), port.getId(), port.getPortUuid(),
                                        portFixedIp.getIpAddress());
                                subnetMap.put(addressBean.getSubnetUuid(), addressBean);
                            }
                            portFixedIpRepository.save(portFixedIp);
                        }
                    }
                    portModel.setPortId(port.getId());
                    if (portMap.containsKey(portModel.getNetworkUuid())) {
                        List<PortModel> models = portMap.get(portModel.getNetworkUuid());
                        models.add(portModel);
                    } else {
                        List<PortModel> models = Lists.newArrayList();
                        models.add(portModel);
                        portMap.put(portModel.getNetworkUuid(), models);
                    }
                }
                logger.info("Start to sync vm port...");
                List<NetworkBean> beans = Lists.newArrayList();
                for (String key : portMap.keySet()) {
                    List<PortModel> portList = portMap.get(key);
                    if (!portList.isEmpty()) {
                        NetworkBean networkBean = networkMap.get(key);
                        List<AddressBean> addresses = Lists.newArrayList();
                        for (PortModel pModel : portList) {
                            AddressBean address = subnetMap.get(pModel.getPortFixedIps().get(0).getSubnetUuid());
                            if (null != address) {
                                address.setAddress(pModel.getPortFixedIps().get(0).getIpAddress());
                                address.setPortId(pModel.getPortId());
                                address.setPortUuid(pModel.getPortUuid());
                                addresses.add(address);
                            }
                        }
                        networkBean.setAddresses(addresses);
                        beans.add(networkBean);
                    }
                }
                // 设置内网所有ip
                cloudServer.setPrivateIps(JSONObject.toJSONString(beans));
                // 保存安全组信息
                if (!StringUtils.isEmpty(server.getGroups())) {
                    logger.info("Start to sync vm security group...");
                    // 删除旧安全组与云主机的关系
                    serverSgroupRepository.removeBySid(cloudServer.getId());
                    List<String> groups = JSONObject.parseArray(server.getGroups(), String.class);
                    for (String group : groups) {
                        SecurityGroup securityGroup = securityGroupRepository.getByName(group, null);
                        if (null != securityGroup) {
                            // 保存云主机和安全组关系
                            serverSgroupRepository.save(new ServerSgroup(cloudServer.getId(), securityGroup.getId()));
                        }
                    }
                    logger.info("End to sync vm security group...");
                }
                if (BocloudStatus.Server.ERROR.name().equals(cloudServer.getStatus())) {
                    cloudServer.setStatus(BocloudStatus.Server.EXCEPTION.name());
                }
                cloudServerRepository.update(cloudServer);
                // 保存资源
                List<String> ips = Lists.newArrayList();
                if (!StringUtils.isEmpty(cloudServer.getPrivateIps())) {
                    List<JSONObject> objects = JSONObject.parseArray(cloudServer.getPrivateIps(), JSONObject.class);
                    for (JSONObject object : objects) {
                        List<Object> addresses = object.getJSONArray("addresses");
                        for (Object address : addresses) {
                            if (StringUtils.isEmpty(
                                    JSONObject.parseObject(JSONObject.toJSONString(address)).getString("address"))) {
                                continue;
                            }
                            ips.add(JSONObject.parseObject(JSONObject.toJSONString(address)).getString("address"));
                        }
                    }
                }
                resource = resourceRepository.query(ResourceCategory.Computer.name(), ResourceCatalog.LOGICAL.name(),
                        cloudServer.getId());
                if (null == resource) {
                    resource = new Resource(ResourceCategory.Computer.name(), ResourceCatalog.LOGICAL.name(),
                            cloudServer.getId(), JSONObject.toJSONString(ips), cloudServer.getOsCategory(),
                            vendor.getId());
                    resource.preSave(cloudServer.getName(), cloudServer.getStatus(), userId, context.getTenant(),
                            context.getProject());
                    resource.setInventory(JSONObject.toJSONString(cloudServer));
                    resourceRepository.save(resource);
                } else {
                    resource.setInventory(JSONObject.toJSONString(cloudServer));
                    resource.setStatus(cloudServer.getStatus());
                    resource.setMenderId(userId);
                    resource.setCreatorId(userId);
                    resource.setGmtModify(new Date());
                    resource.setGmtCreate(new Date());
                    resource.setPrivateIps(JSONObject.toJSONString(ips));
                    resourceRepository.update(resource);
                }
            } else {
                logger.info("Remove the cloned virtual machine failure");
                event = new ResourceEvent(cloudServer.getId(), handler, "失败", userId, new Date(), "VM", vendor.getId(),
                        context.getCatalog());
                cloudServerRepository.remove(cloudServer.getId(), userId);
                // 回收IP
                recycleIp(model.getNetworkBeans(), vendor.getId());
            }
            resourceEventRepository.save(event);
        } catch (Exception e) {
            logger.error("Get error message:", e);
            cloudServerRepository.remove(cloudServer.getId(), context.getTarget());
            // 回收IP
            recycleIp(model.getNetworkBeans(), vendor.getId());
            return new BocloudResult<>(false, cloudServer, "云主机创建失败！");
        }
        return result.isSuccess() ? new BocloudResult<>(true, cloudServer, result.getMessage())
                : new BocloudResult<>(false, cloudServer, result.getMessage());
    }

    /**
     * 更新状态
     *
     * @param server
     * @throws Exception
     */
    public void updateStatus(CloudServer server, String status) throws Exception {
        if (!status.equalsIgnoreCase(BocloudStatus.Server.BUILDING.name())) {
            server.setStatus(status);
            return;
        }
        Long start = System.currentTimeMillis();
        boolean running = true;
        while (running) {
            // 超时和延时
            if (System.currentTimeMillis() - start > 1000 * 60 * 10) {
                server.setStatus(BocloudStatus.Server.EXCEPTION.name());
                running = false;
            }
            BocloudResult result = serverDriver.detail(server.getInstanceId(), vendor.getUuid(), null);
            if (result.isSuccess()) {
                VMBean bean = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), VMBean.class);
                String state = bean.getStatus();
                // 云主机为错误状态
                if (state.equalsIgnoreCase(BocloudStatus.Server.ERROR.name())) {
                    server.setStatus(BocloudStatus.Server.EXCEPTION.name());
                    running = false;
                    break;
                } else if (server.getStatus().equalsIgnoreCase(BocloudStatus.Server.BUILDING.name())) {
                    Thread.sleep(3000L);
                } else {
                    server.setStatus(state);
                    server.setGmtVmCreate(bean.getGmtVmCreate());
                    running = false;
                }
            } else {
                server.setStatus(BocloudStatus.Server.EXCEPTION.name());
                running = false;
            }
        }
    }

    /**
     * 返回随机字符串，同时包含数字、大小写字母
     *
     * @param len 字符串长度，不能小于3
     * @return String 随机字符串
     */
    public static String randomStr(int len) {
        if (len < 3) {
            throw new IllegalArgumentException("字符串长度不能小于3");
        }
        // 数组，用于存放随机字符
        char[] chArr = new char[len];
        // 为了保证必须包含数字、大小写字母
        chArr[0] = (char) ('0' + PwdRandom.uniform(0, 10));
        chArr[1] = (char) ('A' + PwdRandom.uniform(0, 26));
        chArr[2] = (char) ('a' + PwdRandom.uniform(0, 26));

        char[] codes = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I',
                'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd',
                'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y',
                'z'};
        // charArr[3..len-1]随机生成codes中的字符
        for (int i = 3; i < len; i++) {
            chArr[i] = codes[PwdRandom.uniform(0, codes.length)];
        }

        // 将数组chArr随机排序
        for (int i = 0; i < len; i++) {
            int r = i + PwdRandom.uniform(len - i);
            char temp = chArr[i];
            chArr[i] = chArr[r];
            chArr[r] = temp;
        }

        return new String(chArr);
    }

}
