package com.bocloud.ims.service.internal;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudToken;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.ims.common.utils.FastjsonUtils;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * SOC系统配置内部访问
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2019/4/15
 */
@Slf4j
@Component
public class SmsSystemConfigService implements InitializingBean {

	@Autowired
	private ServiceFactory serviceFactory;
	@Autowired
	private CurrentService currentService;

	private static Service service = Service.create(BoCloudService.SMS);
	private Map<String, Object> headers = new HashMap<>(4);

	// 这个接口在sms中没有  sxp:2023.8.29
	public String getByCode(String code) {
		RemoteService remote = serviceFactory.build(service, "/v1/sysconfigs", BoCloudMethod.GET, headers,
				MapTools.simpleMap(Common.CODE, code));
		BocloudResult result = remote.invoke();
		if (result.isFailed() || null == result.getData()) {
			return null;
		}
		return String.valueOf(result.getData());
	}

	public JSONObject getJSONObjectByCode(String code) {
		RemoteService remote = serviceFactory.build(service, "/v1/configs", BoCloudMethod.GET, headers,
				MapTools.simpleMap("codes", code));
		BocloudResult result = remote.invoke();
		if (result.isFailed() || null == result.getData()) {
			return null;
		}
		Object data = result.getData();
		if (data == null) {
			return null;
		}
		String jsonString = JSON.toJSONString(data);
		if (!FastjsonUtils.isJSONObject(jsonString)) {
			log.info("getJSONObjectByCode result is :" + JSON.toJSONString(result));
			return null;
		}

		return JSON.parseObject(jsonString, JSONObject.class);
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
		this.headers.put("source", currentService.getService().getName());
	}
}
