package com.bocloud.ims.service.thread;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.driver.storage.VolumeDriver;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.ims.entity.resource.CloudVendor;
import com.bocloud.ims.entity.resource.Volume;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

/**
 * tce云硬盘创建
 *
 * @author wangyu
 * @version 4.2.1-SNAPSHOT
 * @since 2019/11/6
 */
public class TceVolumeBuilder implements VolumeBuilder {

	private static Logger logger = LoggerFactory.getLogger(OSVolumeBuilder.class);
	private VolumeDriver driver;
	private VolumeModel model;
	private String code;
	private CloudVendor vendor;
	private String region;
	private RequestContext context;

	/**
	 * @param driver
	 * @param model
	 * @param code
	 * @param vendor
	 * @param region
	 * @param context
	 */
	public TceVolumeBuilder(VolumeDriver driver, VolumeModel model, String code, CloudVendor vendor, String region,
			RequestContext context) {
		super();
		this.driver = driver;
		this.model = model;
		this.code = code;
		this.vendor = vendor;
		this.region = region;
		this.context = context;
	}

	@Override
	public BocloudResult<Volume> call() throws Exception {
		return build();
	}

	@Override
	public BocloudResult<Volume> build() {
		BocloudResult result = driver.create(vendor.getUuid(), code, model);
		if (result.isFailed()) {
			Volume volume = new Volume();
			BeanUtils.copyProperties(model, volume);
			return new BocloudResult<>(false, volume, result.getMessage());
		}
		Volume volume = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), Volume.class);
		String status = volume.getStatus();
		volume.setStatus(BocloudStatus.Server.BUILDING.name());
		volume.setRegion(region);
		volume.preSave(context.getTarget(), context.getTarget(), context.getTenant(), vendor.getId(),
				context.getProject());
		volume.setOwnerId(context.getTarget());
		Long start = System.currentTimeMillis();
		boolean running = true;
		while (running) {
			if (System.currentTimeMillis() - start > 1000 * 60 * 5) {
				volume.setStatus(BocloudStatus.Server.EXCEPTION.name());
				running = false;
			}
			result = driver.detail(volume.getVolumeUuid(), vendor.getUuid(), code);
			if (result.isSuccess()) {
				VolumeModel bean = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), VolumeModel.class);
				String state = bean.getStatus();
				if (state.equalsIgnoreCase(status)) {
					try {
						Thread.sleep(3000L);
					} catch (InterruptedException e) {
						logger.error("OSVolumeBuilder get InterruptedException : ", e);
						Thread.currentThread().interrupt();
					}
				} else {
					volume.setStatus(state);
					running = false;
				}
			} else {
				volume.setStatus(BocloudStatus.Server.EXCEPTION.name());
				running = false;
			}
		}
		return new BocloudResult<>(true, volume, "success");
	}
}
